/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var serializer;
var noTextElements =  new Array(
		new Array('table', true), 
		new Array('thead', true), 
		new Array('tfoot', true), 
		new Array('tbody', true), 
		new Array('tr', true) );

document.onclick = document.click =
    function( e )
    {
		//e.preventDefault();
		if( e.button == 0){
            //if (haveloc || (s.rangeCount == 0)){;
			var id = activateSelectedLocation();
			if(id)
				onSelectedLocationChangeByClick();
		}
    };
function getSerializer() {
    if( serializer == null ) serializer = new XMLSerializer();
    return serializer;
}

//----------------------------------------------------------------

function undoAction( action )
{
    switch( action.action )
    {
    case "create":
        if( !removeLocation( action.uuid ) ) return false;
        break;
    case "delete":
        restoreSelection( action.selection );

        if( !createLocation( action.uuid ) ) return false;
        break;
    default:
        alert( "Unknown action '" + action.actionId + "'. Please, fill bug report at http://forge.ispras.ru/projects/reqdb/" );
        return false;
    }
    return true;
}

function convertXPathToPoint( xpath , isStart)
{
    var node = lookupElementByXPath( xpath.node );
    var offset = xpath.offset;
    if( xpath.nodeType == Node.TEXT_NODE )
    {
        var children = node.childNodes;
        for( var i = 0; i < children.length; i++) 
        {
            var child = children[i];
            if( child.nodeType != Node.TEXT_NODE ) continue;
            if( offset < child.nodeValue.length || !isStart && offset == child.nodeValue.length)
            {
                node = child;
                break;
            } else {
                offset -= child.nodeValue.length;
            }
        }
    }
    return { node: node, offset: offset };
}

function restoreSelection( selectionDescriptor )
{
    var selection = window.getSelection();
    selection.removeAllRanges();
    for( var i = 0; i < selectionDescriptor.length; i++ )
    {
        var rangeDescriptor = selectionDescriptor[i];
        var range = document.createRange();
        
        var startPoint = convertXPathToPoint( rangeDescriptor.start, true );
//        alert("range start = " + rangeDescriptor.start.node + " ### " + startPoint.node + " ### " + startPoint.offset );
        range.setStart( startPoint.node, startPoint.offset );
        
        var endPoint = convertXPathToPoint( rangeDescriptor.end, false );
//        alert("range end = " + rangeDescriptor.end.node + " ### " + endPoint.node + " ### " + endPoint.offset );
        range.setEnd( endPoint.node, endPoint.offset );
        
        selection.addRange( range );
    }        
}

function redoAction( action )
{
    switch( action.action )
    {
    case "create":
        restoreSelection( action.selection );

        if( !createLocation( action.uuid ) ) return false;
        break;
    case "delete":
        if( !removeLocation( action.uuid ) ) return false;
        break;
    default:
        alert( "Unknown action '" + action.action + "'. Please, fill bug report at http://forge.ispras.ru/projects/reqdb/" );
        return false;
    }
    return true;
}

//----------------------------------------------------------------

function serializeDocument() {
	log.profile( 'serialize' );

	removeAllLinks();
    deactivateLocations();
    var scripts = removeScripts();
    var res = getSerializer().serializeToString( document );
    injectScripts( scripts );
    addLocationLinks();

	log.profile( 'serialize' );
    
    return res;
}

function serializeDocumentRaw() {
    return getSerializer().serializeToString( document );
}

function injectScripts( scripts ) {
    $( "head" ).append( scripts[0] );
    $( "body" ).append( scripts[1] );
}

function removeScripts() {
    var head = $("#jquery_js").add("#blackbird_js").add("#blackbird_css").add("#edit_js").add("#edit_css");
    head.remove();
    var body = $("#blackbird");
    body.remove();
    return [ head, body ];
}

/**
 * Mark intersections of range and text node range.
 * 
 * new:
 * 
 * <span class="requality_text id_${uuid}">
 *     <a name="${uuid}" id="id_${uuid}" class="requality_id"/>
 *     <span class="requality_link requality_link_(in)?visible">${id}</span>
 *     ...
 * </span>
 * <span class="requality_text id_${uuid}">...</span>
 * 
 * old:
 * 
 * <span class="location_text id_${uuid}"><a name="${uuid}" id="id_${uuid}" class="location_id id_${uuid}"/>...</span>
 * <span class="location_link id_${uuid} location_link_(in)?visible>${id}</span>
 * <span class="location_text id_${uuid}">...</span>
 * <span class="location_text id_${uuid}">...<span class="location_end id_${uuid}"/></span>
 * 
 * @param range    
 * @param textNode
 * @return 1 if intersection of range and textNode is not empty and is not equal to textNode
 * To mark such intersection we add one child node. 
 */
//mark for 
var textStart = false;
function markTextRange( id, range, textNode ) {
    var r = document.createRange();
    r.selectNode(textNode);
    var res = 0;
    var isStart = false;
    var isEnd = false;
    // r is before range: r.end <= range.start
    if( range.compareBoundaryPoints(Range.END_TO_START,r) > 0 ) return 0;
    // r is after range: range.end <= r.start
    if( range.compareBoundaryPoints(Range.START_TO_END,r) < 0 ) return 0;
    switch( range.compareBoundaryPoints(Range.START_TO_START,r) ) {
    	case 1:
    		r.setStart( range.startContainer, range.startOffset );
    		res = 1;
    	case 0:
    		isStart = true;
    }
    switch( range.compareBoundaryPoints(Range.END_TO_END,r) ) {
    	case -1:
    		r.setEnd( range.endContainer, range.endOffset );
    		res = 1;
    	case 0:
    		isEnd = true;
    }
    // if inner text is null, do not count it    
    if( r.toString().trim().length == 0 ) return 0;
    var reqwrap = $("<span/>").attr("class","requality_text id_" + id);
    r.surroundContents( reqwrap.get(0) );
    if( isStart || textStart )
    {
    	textStart = false;
        reqwrap.prepend( $("<a/>").attr("name",id).attr("id","id_"+id).attr("class","requality_id") );
    }
//    if( isEnd )
//    {
//        reqwrap.append($("<span/>").attr("class","location_end id_"+id));
//    }
    return res;
}

function getNoTextFlag(node, array){
	for(var i = 0;i<array.length;i++)
		if(array[i][0]==node)
			return array[i][1];
	return false;
}
/**
 * Mark part of 'node'-subtree that is contained in 'range'
 * 
 * @param id - location unique id
 * @param range
 * @param node
 * @return number of modified ranges 
 */
function markRange( id, range, node ) {
    var skipText = false;
    skipText = getNoTextFlag(node.localName,noTextElements);
    var childNodesCount = node.childNodes.length;
    var tnodes = 0;
    if( childNodesCount > 0 )
    {

        for( var i = 0; i < childNodesCount; i++ )
        {
            var r = document.createRange();
            var child = node.childNodes.item(i);
            if(child == null)
            	continue;
            r.selectNode(child);
            // r is after range: range.end <= r.start

            if( range.compareBoundaryPoints(Range.START_TO_END, r) < 0 ||
            		range.compareBoundaryPoints(Range.END_TO_START, r) > 0)
                continue;
            if( child.nodeType == Node.TEXT_NODE )
            {
            	if(!skipText){
            		var newChildCount = markTextRange( id, range, child );
            		tnodes += newChildCount; 
            		i += newChildCount;
            		childNodesCount += newChildCount;
            	}
            } else {
                tnodes += markRange( id, range, child );
            }
        }
    } else {
    	if(!skipText){
    	    var r = document.createRange();
    		r.selectNode(node);
    		//r is before range: r.start = r.end < range.start
    		if( range.compareBoundaryPoints(Range.END_TO_START,r) > 0 ) 
    		{
    			return 0;
    		}
    		//r is after range: range.end < r.start
    		if( range.compareBoundaryPoints(Range.START_TO_END,r) < 0 )
    		{
    			return 0;
    		}
    		if(!$(node).hasClass("requality_node"))
    			$(node).addClass("requality_node");
    		if(!$(node).hasClass("id_"+id))
    			$(node).addClass("id_"+id);
    		if ($(node)[0].nodeName == "img"){
    			var reqwrap = $("<span/>").attr("class","requality_text id_" + id);
    			r.surroundContents( reqwrap.get(0) );
    			$(node).before( $("<a/>").attr("name",id).attr("id","id_"+id).attr("class","requality_id") );
    		}
    		tnodes = 1;
    	}
    }
    return tnodes;
}

/**
 * Mark selection with tags/styles
 * 
 * @param id - location unique id
 * @returns selection descriptor: array of range descriptors
 * @see createRangeDescriptor
 */
function markSelection( id ) {
    var selection = window.getSelection();
    switch( selection.rangeCount )
    {
    	case 0:
    		alert( "Selection is empty" );
    		return null;
    //case 1:
      //  break;
    	default:
//        alert( "Multipart selection processing is experimental. Please, report any bugs found." );
    }
    var selectionDescriptor = [];
    textStart = true;
    for( var r = 0; r < selection.rangeCount; r++ )
    {
        var changednodes = 0;
        var range = selection.getRangeAt(r);
//        alert( "range.startOffset: " + range.startOffset );
//        alert( "range.endOffset: " + range.endOffset );
        var rangeDescriptor = createRangeDescriptor( range );
        var container = range.commonAncestorContainer;
        if( container.nodeType == Node.TEXT_NODE )
        {
        	// range is a part of text node. Mark intersection.
        	changednodes = markTextRange( id, range, container );
        } else {
        	// Find text node children and mark intersections.
            // Find nodes without text children and mark them also.
        	changednodes = markRange( id, range, container );
        }
        if(changednodes > 0){
        	selectionDescriptor.push( rangeDescriptor );
        }
    }
    selection.collapseToStart();
    return selectionDescriptor;
}

/**
 * This function wrap selection in special requality tags.
 * <br><br><i>Can be used in java</i>
 * @param id UUID
 * @param userId user visible id( <i>For example, 'some.id.01'</i> )
 * @return - selection text
 */
function createLocation( id, reqId ) {
    var selectionDescriptor = markSelection( id );
    if( !selectionDescriptor ) return null;
    addLocationLink( id, reqId );
    var action = { action: "create", uuid: id, selection: selectionDescriptor }; 
    var text = "";
    for( var r = 0; r < selectionDescriptor.length; r++ )
    {
        var rangeDescriptor = selectionDescriptor[r];
        text += rangeDescriptor.text;
    }
    if(selectionDescriptor.length > 0)
    	return [ text, JSON.stringify(action) ];
    return null;
}

function getSelectionText() {
	return window.getSelection().toString().replace("/r","");
}

function addLocationLinks() {
//	var locations = getDocumentLocationIds();
//	if( locations.length == 0 ) return;
//	var id2links = getLocationsToRequirementMap( locations );
//	for( var i = 0; i < id2links.length; i++ )
//	{
//		var chunk = id2links[i];
//		for( var j = 1; j < chunk.length; j++ )
//		{
//			addLocationLink( chunk[0], chunk[j] );
//		}
//	}		
}

function addLocationLink( id, link ) {
    $("#id_"+id).after($("<span>"+link+"</span>").addClass("requality_link"));
}

function removeOnEmpty(node){
	var r = document.createRange();
    r.selectNode(node);
    if(!isRangeSelectable(node,r))
    	node.remove();
}
function removeLocationLink( id, link ) {
    $("#id_"+id).parent().children(".requality_link :contains('"+link+"')").remove();
}

function removeAllLinks() {
    $(".requality_link").remove();
}
function getSelectedLocationUUIDFromNode(f, range){
	var id = null;
	if( !f )
		return null;
	var node = $(f);
	if(node.hasClass("requality_text") ||
			node.hasClass("requality_node"))
				id = node.attr("class");
	if(!id){
		node.find(".requality_text").each(
			function(){
					var ssid = this.getAttribute("class");
					//skip to next
					if(range){
						var r = document.createRange();
						r.selectNode(this);
						if( range.compareBoundaryPoints(Range.END_TO_START,r) > 0 ) return true;
						if( range.compareBoundaryPoints(Range.START_TO_END,r) < 0 ) return true;
					}
			    	if( ssid ){
			    		id = ssid;
			    		return false;
			    	}
					return true;
	   			}
	   		);
	}
	return id;
}

function getSelectedLocationUUID() {
    var s = window.getSelection();
	var id = null;
	var parentSel = false;
	for( var i = 0; i < s.rangeCount; i++ ){
		range = s.getRangeAt(i);
		if(range.commonAncestorContainer){
			id = getSelectedLocationUUIDFromNode(range.commonAncestorContainer, range);
			if(id)
				break;
			if(!parentSel){
				id = $(range.commonAncestorContainer).parents(".requality_text").attr("class");
				parentSel = true;
			}
		}else{
			id = getSelectedLocationUUIDFromNode(range.startContainer, range);
			if(id)
				break;
			id = getSelectedLocationUUIDFromNode(range.endContainer, range);
			if(id)
				break;
			if(!parentSel){
				id = $(range.startContainer).parents(".requality_text").attr("class");
				parentSel = true;
			}
		}
	}
	if(s.rangeCount == 0){
		if (!id)
			id = getSelectedLocationUUIDFromNode(s.focusNode);
		if(!id)
			id = getSelectedLocationUUIDFromNode(s.anchorNode);
	}
    if(!id)
    	return null;
    var fst = id.indexOf("id_");
    var spc = id.indexOf(" ", fst);
    if( spc == -1 ) spc = id.length;
    id = id.substring(fst+3, spc);
    return id;
}

function getDocumentLocationIds()
{
	var res = [];
	$(".requality_id").each(
		function()
		{
			var id = this.getAttribute("id");
			res.push( id.substring(3) );
		}
	);
	return res;
}

function removeLocation( id ) {
	var t = $("span.requality_text.id_" + id );
	//get parents of nodes to be removed
	oldparents = [];
	t.find("a.requality_id#id_"+id).remove();
	t.each(function(){
		oldparents.push(this.parentNode);
	});
	//t.remove( ".requality_link.id_" + id );
    var selectedNodes = [];
    var selectedNodesText = [];
    var first=-1,last=-1;
    var ftext, ltext;
    $(".requality_node.id_"+id).each(function(){
		var nde = $(this);
		nde.removeClass("id_"+id);
		if(nde.attr("class").indexOf("id_")<0)
			nde.removeClass("requality_node");
		nde.removeClass("requality_node_active");
	});
	t.contents().each(
		function(i){
			selectedNodes.push( this );
		}
	);
	// move text from inside of span before it
	t.contents().unwrap();
	t.remove();
	var selectionDescriptor = [];
    var range = document.createRange();
    var lasttext = -1,lasttextnode;
    for( var i = 0; i < selectedNodes.length; i++ )
    {
	    var node = selectedNodes[i];
	    if (node.nodeType == Node.TEXT_NODE){
	    	lasttextnode = node;
	    	if (node.textContent.length == 0){
	    		node.remove();
	    		continue;
	    	}
	    	if (node.previoussibling!=null && node.previoussibling.nodeType == Node.TEXT_NODE){
	    		var nodelen = node.textContent.length;
	    		node.previoussibling.textContent+=node.textContent;
	    		range.selectNode(node.previoussibling);
	    		range.setStartOffset(node.previoussibling.textContent.length-nodelen);
	    		range.setEndOffset(node.previoussibling.textContent.length);
	    		lasttextnode = node.previoussibling;
	    		node.remove();
	    		var rangeDescriptor = createRangeDescriptor(range);
	            selectionDescriptor.push( rangeDescriptor );
	            continue;
	    	}
	    	lasttext = i;
	    }
        range.selectNode(node);
        var rangeDescriptor = createRangeDescriptor(range);
        selectionDescriptor.push( rangeDescriptor );
    }
    if (lasttext!=-1 && lasttextnode.nextsibling!=null && lasttextnode.nextsibling.nodeType == Node.TEXT_NODE){
		lasttextnode.textContent+=lasttextnode.nextsibling.textContent;
		lasttextnode.nextsibling.remove();
	}
    //for( var i = 0;i<oldparents.length;i++)
    	//oldparents[i].normalize();
	var action =  { action: "delete", uuid: id, selection: selectionDescriptor };
	return JSON.stringify( action );
}

function deactivateLocations() {
    $( ".requality_text" ).removeClass( "requality_text_active" );
    $( ".requality_node" ).removeClass( "requality_node_active" );
}

function setSelection( node )
{
    var selection = window.getSelection();
    var range = document.createRange();
    range.setStartAfter( node );
    range.setEndAfter ( node ); 
    if(selection.clear)
    	//chrome
    	selection.clear();
    if(selection.removeAllRanges)
    	selection.removeAllRanges();
    selection.addRange( range );
}

function activateLocation( id, scroll ) {
    deactivateLocations();
    
    var location = $( "a#id_" + id );
    if( location.size() == 0 )
    {
        return;
    }
    
    $( ".requality_text.id_"+id ).addClass( "requality_text_active" );
    $( ".requality_node.id_"+id ).addClass( "requality_node_active" );
    
    if( scroll )
    {
        window.scrollTo( 0, location.offset().top - window.innerHeight / 2 );
    }

    setSelection( location.get(0) );
}

function activateSelectedLocation() {
    var id = getSelectedLocationUUID();
    if( id != null )
    {
        activateLocation( id, true );
        return id;
    } else {
        deactivateLocations();
    }
    return null;
}

/**
 * Search for aString with given options.
 * <br><br><i>Can be used in java</i>
 */
function findText( aString
                 , aCaseSensitive
                 , aBackwards
                 , aWrapAround
                 )
{
    return window.find( aString
                      , aCaseSensitive
                      , aBackwards
                      , aWrapAround
                      , false
                      , false
                      , false
                      );
}

/**
 * Selecting requality Location node and makes it orange.
 * <br><br><i>Can be used in java</i>
 */
function selectNode() {
    alert( 'selectNode' );
}

function getBrowserInfo() {

	var appName= navigator.appName;
    var appAgent= navigator.userAgent;
    var appVersion = navigator.appVersion;
    return "Browser: " + appName + "; UserAgent: " + appAgent+"; Version: "+appVersion;
}
function isTextRangeSelectable( range, textNode ) {
    var r = document.createRange();
    r.selectNode(textNode);
    var res = 0;
    var isStart = false;
    var isEnd = false;
    if( range.compareBoundaryPoints(Range.END_TO_START,r) > 0 ) return false;
    if( range.compareBoundaryPoints(Range.START_TO_END,r) < 0 ) return false;
    switch( range.compareBoundaryPoints(Range.START_TO_START,r) ) {
    case 1:
        r.setStart( range.startContainer, range.startOffset );
        res = 1;
    case 0:
    	isStart = true;
    }
    switch( range.compareBoundaryPoints(Range.END_TO_END,r) ) {
    case -1:
    	r.setEnd( range.endContainer, range.endOffset );
        res = 1;
    case 0:
        isEnd = true;
    }
    if( r.toString().trim().length == 0 ) return false;
    return (res > 0) ? true : false;
}
function isRangeSelectable(range, node ) {
    var r = document.createRange();
    var skipText = false;
    skipText= getNoTextFlag(node.localName,noTextElements);
    var childNodesCount = node.childNodes.length;
    //if it is location here, then we cannot add locations here
    var isSelect = false;
    if( childNodesCount > 0 )
    {
        for( var i = 0; i < childNodesCount; i++ )
        {
            var child = node.childNodes.item(i);
            r.selectNode(child);
            if( range.compareBoundaryPoints(Range.START_TO_END, r) < 0 ||
            		range.compareBoundaryPoints(Range.END_TO_START, r) > 0)
                continue;
            
            if( child.nodeType == Node.TEXT_NODE )
            {
            	if(!skipText){
            		if(isTextRangeSelectable(range, child ))
            			isSelect = true;
            	}
            } else {
                if (isRangeSelectable(range, child ))
                	isSelect = true;
            }
        }
        return isSelect;
    } else {
    	if(!skipText){
    		tnodes = 1;
    		r.selectNode(node);
    		if( range.compareBoundaryPoints(Range.END_TO_START,r) > 0 ) 
    			return false;
    		if( range.compareBoundaryPoints(Range.START_TO_END,r) < 0 )
    			return false;
    		return node.nodeName.toLowerCase() == "img";
    	}
    }
    return false;
}
function isSelectable() {
	var selection = window.getSelection();
    switch( selection.rangeCount )
    {
    case 0:
        return false;
    }
    var isSelect = false;
    for( var r = 0; r < selection.rangeCount; r++ )
    {
        var range = selection.getRangeAt(r);
        var container = range.commonAncestorContainer;
        if( container.nodeType == Node.TEXT_NODE )
        {
        	if(isTextRangeSelectable(range, container ))
        		isSelect = true;
        } else {
        	if(isRangeSelectable(range, container ))
        		isSelect = true;
        }
    }
	return isSelect;
}

/**
 * <i>Can be used in java</i>
 * @return - <b>true</b> if selected node can be unwrapped
 */
function isRemovable() {
    var uuid = getSelectedLocationUUID();
    return uuid != null && uuid.length > 0;
}

function createXPathFromElement(elm) { 
    var allNodes = document.getElementsByTagName('*');
    var nodeType = Node.ELEMENT_NODE;
    if( elm.nodeType == Node.TEXT_NODE ) // text node
    {
        elm = elm.parentNode;
        nodeType = Node.TEXT_NODE;
    }
    for (segs = []; elm && elm.nodeType == Node.ELEMENT_NODE; elm = elm.parentNode) 
    {
        if (elm.hasAttribute('id')) { 
                var uniqueIdCount = 0; 
                for (var n=0;n < allNodes.length;n++) { 
                    if (allNodes[n].hasAttribute('id') && allNodes[n].id == elm.id) uniqueIdCount++; 
                    if (uniqueIdCount > 1) break; 
                }; 
                if ( uniqueIdCount == 1) { 
                    segs.unshift('id("' + elm.getAttribute('id') + '")'); 
                    return { node: segs.join('/'), nodeType: nodeType }; 
                } else { 
                    segs.unshift("x:" + elm.localName.toLowerCase() + '[@id="' + elm.getAttribute('id') + '"]'); 
                } 
//        } else if (elm.hasAttribute('class')) { 
//            segs.unshift("x:" + elm.localName.toLowerCase() + '[@class="' + elm.getAttribute('class') + '"]'); 
        } else { 
            for (i = 1, sib = elm.previousSibling; sib; sib = sib.previousSibling) { 
                if (sib.localName == elm.localName)  i++; }; 
                segs.unshift("x:" + elm.localName.toLowerCase() + '[' + i + ']'); 
        }; 
    }; 
    return segs.length ? { node: '/' + segs.join('/'), nodeType: nodeType } : null; 
}; 

function resolver( prefix )
{
    return "http://www.w3.org/1999/xhtml";
}

function lookupElementByXPath(path) { 
    var evaluator = new XPathEvaluator();
    var result = evaluator.evaluate(path, document, resolver,XPathResult.ANY_TYPE, null);
    switch( result.resultType )
    {
    case XPathResult.UNORDERED_NODE_ITERATOR_TYPE:
        while( true )
        {
            var node = result.iterateNext();
            if( !node ) break;
            return node;
        }
    }
    return null;
}

function setOffset( elm, offset, point )
{
    point.offset = offset;
//alert("setOffset, offset = " + offset + " point.nodeType = " + point.nodeType );
    if( point.nodeType != 3 ) return;
//alert("setOffset, text node" );
    var parent = elm.parentNode;
    for( var sib = elm.previousSibling; sib; sib = sib.previousSibling )
    {
//alert("setOffset, previous sibling" );
        if( sib.nodeType != 3 ) continue;
//alert("setOffset, previous sibling is text node" );
        point.offset += sib.nodeValue.length;
//alert("setOffset, offset = " + point.offset );
    }
}

function createRangeDescriptor( range )
{
    var start = createXPathFromElement( range.startContainer );
    setOffset( range.startContainer, range.startOffset, start );
    var end = createXPathFromElement( range.endContainer );
    setOffset( range.endContainer, range.endOffset, end );
    var text = getSerializer().serializeToString( range.cloneContents() );
    return { start: start, end: end, text: text };
}
    
function propNames( obj )
{
    if( !obj ) return "obj is undefined";
    var s = "/*" + obj.toString() + ":*/{";
    var comma = false;
    for( p in obj ) { 
        if( comma ) s += ",";
        comma = true;
        try 
        {
            s += "|" + p + "=" + obj[p];
        } 
        catch(e)
        {
            s += "|" + p + "=<not shown>";
        }
    }
    s += "\n}";
    return s;
}

function inspect( obj ) {
    alert( propNames( obj ) );
}

function inspectRange( tip, range )
{
    var text = getSerializer().serializeToString( range.cloneContents() );
    alert( tip + text );
}

function inspectNode( tip, node )
{
    var range = document.createRange();
    range.selectNode(node);
    inspectRange( tip, range );
}

function inspectRangeNode( tip, range, node )
{
    var rangeText = getSerializer().serializeToString( range.cloneContents() );

    var nodeRange  = document.createRange();
    nodeRange.selectNode(node);
    var nodeText = getSerializer().serializeToString( nodeRange.cloneContents() );

    alert( tip + " <" + rangeText + "> <" + nodeText + ">" );
}

function inspectAction( action )
{
    inspect( action.selection[0] );
    inspect( action.selection[0].start );
    inspect( action.selection[0].end );
    var str = "action:"+action.action+"\n"
            + "uuid:"+action.uuid+"\n"
            + "selection:" + selectionDescriptorToString( action.selection );
    alert( str );
}

function selectionDescriptorToString( selection )
{
    var res = "";
    for( var i = 0; i < selection.length; i++ )
    {
        res += rangeDescriptorToString( selection[i] )+"\n";
    }
    return res;
}

function rangeDescriptorToString( range )
{
    var res = "start: "+pointToString( range.start ) + "\n"
            + "end: "+pointToString( range.end ) +"\n"
            + "text:"+range.text;
    return res;
}

function pointToString( point )
{
    var res = "{ node:"+point.node
            + ", nodeType:"+point.nodeType
            + ", offset:"+point.offset+" }";
    return res;
}
function tree( node )
{
	var s = "";
	switch( node.nodeType )
	{
	case Node.ELEMENT_NODE:
		s += "<"+node.nodeName;
		s += " class=" + node.getAttribute("class") + ">";
		for( var i = 0; i < node.childNodes.length; i++ )
		{
			s += "\n"+tree( node.childNodes.item(i) );
		}
		s += "\n</"+node.nodeName+">";
		break;
	case Node.TEXT_NODE:
		s += node.nodeValue;
	}
	return s;
}	

addLocationLinks();
setInitialLocation();
reportDocumentLoaded();
