package com.unitesk.requality.eclipse.handlers;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateTestPurposeHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        Requirement req = (Requirement) nodes.get(0);
        String testPurposeId = RequalityCLI.getNextId(req, TestPurpose.TYPE_NAME, req.getNameOrId()+"_T");
        try {
            if(!req.getTreeDB().getTreeLogic().canBeChildOfExc(TestPurpose.TYPE_NAME, req)){
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Create Element",
                        "Element of type '" + splitWords(TestPurpose.TYPE_NAME)
                        + "' cannot be created in " + req.getUserFriendlyName());
                
                return null;
            }
        } catch (PastingIntoLeafException e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Create Node",
                    "Can't add Test Purpose to "+splitWords(req.getType())+" with non-Test Purpose child");
            return null;
        }
        req.getTreeDB().startTransaction("Create Test Purpose");
        TestPurpose tc = (TestPurpose) req.createChild(testPurposeId,
                TestPurpose.TYPE_NAME);
        tc.putAttribute(new Attribute(AttributeType.STRING,
                TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
        tc.putAttribute(new Attribute(AttributeType.STRING,
                TestPurpose.ATTR_STATUS, "in process"));
        tc.saveAttributes();
        req.getTreeDB().commit();
        Activator.getDefault().setActiveNode(tc, TestPurpose.TYPE_NAME);
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if (nodes == null || (nodes.size() > 1)) {
            return false;
        }
        if(nodes.size() > 1)
            return false;
        if (!nodes.get(0).getTreeDB().getTreeLogic().
                canBeChildOf(TestPurpose.TYPE_NAME, nodes.get(0)))
                return false;
        return true;
    }
    private String splitWords(String s) {
        if (s == null)
            return null;
        for (int i = 1; i < s.length(); i++) //s.length() is variable
            if (s.charAt(i - 1) >= 'a' && s.charAt(i - 1) <= 'z' &&
                s.charAt(i) >= 'A' && s.charAt(i) <= 'Z') {
                s = s.substring(0, i) + " " + s.substring(i);
            }
        return s;
    }
}
