package com.unitesk.requality.eclipse.handlers.location;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.views.properties.PropertySheet;
import org.eclipse.ui.views.properties.tabbed.ISection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class RemoveLocationFromRequirementHandler extends AbstractHandler {

    private ILocationTab getTab() {
        IWorkbenchPart part = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActivePart();
        if (part instanceof PropertySheet) {
            IPage page = ((PropertySheet) part).getCurrentPage();
            if (page instanceof TabbedPropertySheetPage) {
                ISection[] sections = ((TabbedPropertySheetPage) page).getCurrentTab().getSections();
                if (sections != null && sections.length == 1 && sections[0] instanceof ILocationTab) {
                    return (ILocationTab) sections[0];
                }
            }
        }
        return null;
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        ILocationTab tab = getTab();
        if (tab == null) {
            return null;
        }
        Location loc = tab.getLocation();
        Requirement req = tab.getRequirement();
        if (loc == null) {
            return null;
        }
        //req == null - delete all links
        if(req == null){
        	if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete Location",
                    "Delete selected Location?")) {
                return null;
        	}
            loc.getTreeDB().startTransaction("Delete Location");
            Requirement[]reqs = loc.getRequirements();
            for(Requirement rq : reqs)
                CommonRemoveLocation.RemoveLocation(loc, rq, null, true);
            loc.getTreeDB().commit();
        }else{
            CommonRemoveLocation.RemoveLocation(loc, req, null, false);
        }
        tab.refreshLocations();
        return null;
    }

    @Override
    public boolean isEnabled() {
        ILocationTab tab = getTab();
        if (tab == null || tab.getLocation() == null) {
            return false;
        }
        return true;
    }

}
