/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateReportDataHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        TreeNode node = nodes.get(0);
        String id = RequalityCLI.getNextId(node, ReportData.TYPE_NAME, null);
        // TODO report undo
        node.getTreeDB().startTransaction("Create Report Data");
        try {
            if(!node.getTreeDB().getTreeLogic().canBeCreatedInExc(ReportData.TYPE_NAME, id, node)){
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Element of type '" + splitWords(ReportData.TYPE_NAME) 
                        + "' cannot be created in " + node.getUserFriendlyName());
                return null;
            }
        } catch (PastingIntoLeafException e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                    "Can't add "+splitWords(ReportData.TYPE_NAME)+" to "+splitWords(node.getType())+" with "
                    + "Test Purpose child");
            return null;
        } catch (SameIdExistsInChildren e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                    "Element with same id exists in " + 
                    node.getUserFriendlyName());
            return null;
        } catch (IdEmpty e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                    "Element id is empty");
            return null;
        } catch (IdContainsWrongChars e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                    "Element id contains wrong character - " 
                    + e.getWrongCharacter());
            return null;
        }
        ReportData newReportData = (ReportData) node.createChild(id,
                ReportData.TYPE_NAME);
        newReportData.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_TEMPLATE, Activator.getDefault().getReportTemplates().keySet().iterator().next()));
        newReportData.saveAttributes();
        node.getTreeDB().commit();
        Activator.getDefault().setActiveNode(newReportData, ReportData.TYPE_NAME);
        return null;
    }

    @Override
    public boolean isEnabled() {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return false;
        TreeNode node = nodes.get(0);
        if (!node.getType().equals(ReportFolder.TYPE_NAME)) {
            return false;
        }
        return true;
    }

    private String splitWords(String s) {
        if (s == null)
            return null;
        for (int i = 1; i < s.length(); i++) //s.length() is variable
            if (s.charAt(i - 1) >= 'a' && s.charAt(i - 1) <= 'z' &&
                s.charAt(i) >= 'A' && s.charAt(i) <= 'Z') {
                s = s.substring(0, i) + " " + s.substring(i);
            }
        return s;
    }
}
