/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.IUIStateChangeListener;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class RequalityCNF extends CommonNavigator implements
        ITabbedPropertySheetPageContributor {
    public final static String ID = "com.unitesk.requality.views.RequalityExplorer";
    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(final TreeNode node, final TreeNode oldParent) {
        	UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	RequalityCNF.this.refresh(node.getParent());
                	RequalityCNF.this.refresh(oldParent);
                	RequalityCNF.this.refresh(node);
                    return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

        @Override
        public void deleted(final NodeDesc nodeDesc) { //TODO: sync? Job?

            UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	try{
                		RequalityCNF.this.refresh(db.getNode(nodeDesc.getParentUUId()));
                	}catch(RuntimeException re){
                		//node not fount
                	}
                    return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

        private List<TreeNode> dirtyNodes = new ArrayList<TreeNode>();

        @Override
        public void changed(final TreeNode node) {
            if (dirtyNodes.contains(node)) {
                return;
            }
            dirtyNodes.add(node);
            UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	dirtyNodes.remove(node);
                	RequalityCNF.this.updateTitle();
                	RequalityCNF.this.refresh(node);
                	return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

        @Override
        public void created(final TreeNode node) {
        	UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	RequalityCNF.this.refresh(node.getParent());
                	return Status.OK_STATUS;
                }
            };
            job.schedule();
        }
    };
    private TreeDB db = null;
    final private IUIStateChangeListener uiListener = new IUIStateChangeListener() {
        @Override
        public void newActiveNodes(List<TreeNode> nodes) {
            if (nodes == null) {
                UIJob job = new UIJob("update selection") {
                    @Override
                    public IStatus runInUIThread(IProgressMonitor monitor) {
                        RequalityCNF.this.getCommonViewer().setSelection(StructuredSelection.EMPTY);
                        return Status.OK_STATUS;
                    }
                };
                job.schedule();
                return;
            }
            Object[] objs = ((IStructuredSelection) getCommonViewer()
                    .getSelection()).toArray();
            if(objs!=null && objs.length == nodes.size()){
                boolean stop = false;
                for(int i = 0; i<objs.length && !stop; i++)
                    stop = nodes.contains(objs[i]);
                if(stop)
                    return;
            }
            if(nodes == null || nodes.size() == 0)
                return;
            if(db != null)
                db.removeListener(treeChangeListener);
            db = nodes.get(0).getTreeDB();
            db.addListener(treeChangeListener);
            List<TreeNode>sel = new LinkedList<TreeNode>();
            for(TreeNode node:nodes){
                if(node.getTreeDB() != db)
                    continue;
                Object parentElement = node.getParent();
                getCommonViewer().refresh(node.getParent());
                getCommonViewer().expandToLevel(parentElement, 1);
                sel.add(node);
            }
            StructuredSelection elements = new StructuredSelection(sel);
            selectReveal(elements);
        }
    };

    final private ISelectionChangedListener treeSelection = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (event.getSelection() instanceof IStructuredSelection) {
                Iterator<Object> it = ((IStructuredSelection) event.getSelection()).iterator();
                TreeDB db2 = null;
                SortedMap<String, List<TreeNode>> nodes = new TreeMap<String, List<TreeNode>>();
                while(it.hasNext()){
                    Object obj = it.next();
                    if (obj instanceof TreeNode) {
                        TreeNode node = (TreeNode) obj; 
                        if (node == null)
                            continue;
                        if(db2 == null)
                            db2 = node.getTreeDB();
                        //possible missunderstood here
                        if(db2 != node.getTreeDB())
                            continue;
                        if(db != null)
                            db.removeListener(treeChangeListener);
                        if(node!=null)
                            db = node.getTreeDB();
                        db.addListener(treeChangeListener);
                        if(!nodes.containsKey(node.getType()))
                            nodes.put(node.getType(), new LinkedList<TreeNode>());
                        nodes.get(node.getType()).add(node);
                    }
                }
                Activator.getDefault().setActiveNodes(nodes);
            }
        }
    };

    public RequalityCNF() {
        super();
    }

    @Override
    public void dispose() {
        super.dispose();
        getCommonViewer().removePostSelectionChangedListener(treeSelection);
        Activator.getDefault().removeChangeListener(uiListener);
        if(db != null)
            db.removeListener(treeChangeListener);
    }

    @Override
    public void createPartControl(Composite aParent) {
        super.createPartControl(aParent);
        Activator.getDefault().addChangeListener(uiListener);
        getCommonViewer().addPostSelectionChangedListener(treeSelection);
        getCommonViewer().setSorter(new RequalitySorter());
    }

    // This is danger hack
    @Override
    protected CommonViewer createCommonViewerObject(Composite aParent) {
        CommonViewer cv = new CommonViewer(getViewSite().getId(), aParent,
                SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
        return cv;
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class adapter) {
        if (adapter == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return super.getAdapter(adapter);
    }
    public static synchronized List<TreeNode> getSelectedNodes() {
        IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (activeWorkbenchWindow == null)
            return null;
        ISelection selection = activeWorkbenchWindow.getSelectionService()
                .getSelection();

        if (!(selection instanceof IStructuredSelection)) {
            return null;
        }
        IStructuredSelection selected = (IStructuredSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }
        List<TreeNode> nodes = new ArrayList<TreeNode>();
        List<Object> objList = (List<Object>)selected.toList();
        for(Object obj : objList){
            if (obj instanceof TreeNode) {
                TreeNode node = (TreeNode) obj;
                nodes.add(node);
            } else if (obj instanceof IFile) {
                TreeNode node = TreesTracker.getNodeByResource((IFile) obj); 
                nodes.add(node);
            }
        }
        if(nodes.size() > 0)
            return nodes;
        return null;
    }
    public void refresh(TreeNode tn){
        if(tn == null)
            return;
        if(this.getCommonViewer() != null){
            try{
                this.getCommonViewer().refresh(tn);
            }
            catch(Exception e){
                //TODO fix unnecessary refresh
            }
        }
    }
    @Override
    protected Object getInitialInput() {
        if(super.getInitialInput()==null) {
            return ResourcesPlugin.getWorkspace().getRoot();
        }
        return super.getInitialInput();
    }
}
