package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.IContributedContentsView;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.ITreeLogic.TreeLogicException;
import com.unitesk.requality.core.ITreeLogic.WrongProjectException;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.NodeMapper;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeLogic;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.handlers.DeleteSelection;
import com.unitesk.requality.eclipse.handlers.copypaste.CopyHandler;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipseCopyHandler;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipsePasteHandler;
import com.unitesk.requality.eclipse.handlers.copypaste.PasteIntoHandler;
import com.unitesk.requality.eclipse.handlers.location.CommonRemoveLocation;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodesFromList;
import com.unitesk.requality.eclipse.ui.dialogs.SelectTestPurposeDialog;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public abstract class JSEditor extends EditorPart implements ISelectionProvider {

    public static final int EDIT_MODE_REQ_RO = 0x01;
    public static final int EDIT_MODE_TP_RO = 0x02;
    private Browser browser;
    protected File html;
    private TreeNode node;
    private TreeNode currentSelection;
    private TreeDB db;
    private UniEditorOutlinePage uniEditorOutlinePage;
    private int editMode = 0;
    
    private String getSelectionText(boolean process) {
        String res = (String) browser.evaluate("return getSelectionText();");
        if (res == null)
            return "";
        if(!process)
            return res;
        StringTokenizer st = new StringTokenizer(res, "\n ");
        res = "";
        while (st.hasMoreTokens()) {
            if (!res.isEmpty()) {
                res += " ";
            }
            res += st.nextToken();
        }
        return res;
    }
    private void setSelectionText(String text) {
    	browser.execute("setSelectionText('"+text.replace("\r", "\\r").replace("\n","\\n")+"');");
    }
    final class BrowserKeyListener implements KeyListener {
        @Override
        public void keyReleased(KeyEvent e) {
            if ((e.stateMask & SWT.CTRL) != 0 && ((e.stateMask & SWT.SHIFT) != 0)) {
                if (e.keyCode == 'c')
                {
                    String textData = getSelectionText(false);
                    if(textData == null || textData.equals(""))
                        return;
                    Clipboard clipboard = new Clipboard(getSite().getShell().getDisplay());
                    TextTransfer textTransfer = TextTransfer.getInstance();
                    Transfer[] transfers = new Transfer[]{textTransfer};
                    Object[] data = new Object[]{textData};
                    clipboard.setContents(data, transfers, DND.CLIPBOARD);
                    clipboard.dispose();
                }
                if (e.keyCode == 'v')
                {
                	Clipboard clipboard = new Clipboard(getSite().getShell().getDisplay());
                    TextTransfer textTransfer = TextTransfer.getInstance();
                    Object o = clipboard.getContents(textTransfer);
                    clipboard.dispose();
                    String textData = (String)o;
                    if(textData == null || textData.equals(""))
                        return;
                    setSelectionText(RequalityCLI.htmlEscape(textData));
                   
                }
            }
        }

        @Override
        public void keyPressed(KeyEvent e) {
        }
    }

    private ITreeStateListener treeStateListener = new ITreeStateListener() {
        
        @Override
        public void open(TreeDB db) {
            // TODO Auto-generated method stub
            //add tree change listener
            db.addListener(treeChangeListener);
        }
        
        @Override
        public void close(TreeDB db) {
            //remove tree change listener
            db.removeListener(treeChangeListener);
            if(!JSEditor.this.db.equals(db)) {
                return;
            }
            closeEditor();
        }

        @Override
        public void delete(IProject proj) {
            IProject project = TreesTracker.getTracker().getProject(db);
            if (!project .equals(proj)) {
                return;
            }
            closeEditor();
        }

    };

    private ISelectionChangedListener outlineSelectionChangeListener = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            IStructuredSelection selection = (IStructuredSelection) event
                    .getSelection();
            if (selection.isEmpty()) {
                return;
            }
            TreeNode sel = (TreeNode) selection.getFirstElement();
            if(selection instanceof NodeEditorInput)
            {
            	TreeNode dat = (TreeNode)((NodeEditorInput)selection).getData();
            	if(dat!=null)
            		sel = dat;
            }
            setUniEditorFocus(sel.getUUId());
            currentSelection = sel;
            // synchronization with Requality Explorer
            IViewPart view = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .findView(RequalityCNF.ID);
            if (view instanceof CommonNavigator) {
                CommonNavigator navigator = (CommonNavigator) view;
                if (navigator.isLinkingEnabled()) {
                    navigator.setLinkingEnabled(true);
                }
            }
        }
    };

    public TreeNode getCurrentSelection() {
        if(currentSelection == null) {
            currentSelection = targetNode;
        }
        return currentSelection;
    }

    public String getSelectionText() {
        String res = (String) browser.evaluate("return getSelectionText();");
        if (res == null)
            return "";
        StringTokenizer st = new StringTokenizer(res, "\n ");
        res = "";
        while (st.hasMoreTokens()) {
            if (!res.isEmpty()) {
                res += " ";
            }
            res += st.nextToken();
        }
        return res;
    }
    
    public void setSelection(TreeNode node) {
        if(node == null){
            currentSelection = node;
            return;
        }
        setUniEditorFocus(node.getUUId());
        currentSelection = node;
        if (uniEditorOutlinePage != null
                && !uniEditorOutlinePage.getControl().isDisposed()) {
            uniEditorOutlinePage.setSelection(new StructuredSelection(node));
        }
    }

    public boolean findText(String aString, boolean aCaseSensitive,
            boolean aBackwards, boolean aWrapAround) {
        return (Boolean) browser.evaluate("return findText('" + aString
                + "'," + aCaseSensitive + "," + aBackwards + "," + aWrapAround
                + ");");
    }

    private Map<UUID, NodeDesc> changedNodes;
    private Map<UUID, Integer> changedNodesCount;
    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            refresh(node.getParent());
            refresh(oldParent);
        }

        @Override
        public void deleted(NodeDesc nodeDesc) {
            if(!db.getStorage().canLoad(JSEditor.this.node.getUUId())) { // current node was deleted
                closeEditor();
                return;
            }
            if(db.getStorage().canLoad(nodeDesc.getParentUUId())) {
                refresh(db.getNode(nodeDesc.getParentUUId()));
            }
        }

        @Override
        public void changed(TreeNode nde) {
            //if we refresh parent, node will be refreshed
        	String qid = nde.getQualifiedId(),
        		   rootqid = node.getQualifiedId();
        	if(!qid.contains(rootqid))
        		return;
            TreeNode n = nde.getParent();
            if(n == null || qid.equals(rootqid))
                n = nde;
            refresh(n);
        }

        @Override
        public void created(TreeNode nde) {
        	String qid = nde.getQualifiedId(),
         		   rootqid = node.getQualifiedId();
         	if(!qid.contains(rootqid))
         		return;
             TreeNode n = nde.getParent();
             if(n == null || qid.equals(rootqid))
                 n = nde;
             refresh(n);
        }
    };
    //Check if id is valid
    private static boolean reqIdValidExc(TreeNode req, TreeNode parent, String text) throws SameIdExistsInChildren, IdEmpty, IdContainsWrongChars{
        if(req == null && parent == null) return false;
        TreeLogic tlogic;
        if(req == null){
            tlogic = parent.getTreeDB().getTreeLogic();
        }else{
            tlogic = req.getTreeDB().getTreeLogic();
        }
        return tlogic.isChildIdValidExc(parent, req, text);
    }
    private static boolean reqIdValid(TreeNode req, TreeNode parent, String text){
        try{
            return reqIdValidExc(req, parent, text);
        }
        catch(TreeLogicException exc){
            return false;
        }
    }
    //Check if child can be added to parent
    private static boolean reqChildValidExc(TreeNode req, TreeNode parent, String name) throws SameIdExistsInChildren, IdEmpty, IdContainsWrongChars, WrongProjectException, PastingIntoLeafException{
        if(req == null && parent == null) return false;
        TreeLogic tlogic;
        if(req == null){
            tlogic = parent.getTreeDB().getTreeLogic();
        }else{
            tlogic = req.getTreeDB().getTreeLogic();
        }
        return tlogic.isChildIdValidExc(parent, req, name) 
                && tlogic.canBeParentOfExc(parent, req) 
                && tlogic.canBeChildOfExc(req, parent);
    }
    private static boolean reqChildValid(TreeNode req, TreeNode parent, String name){
        try{
            return reqChildValidExc(req, parent, name);
        }
        catch(TreeLogicException exc){
            return false;
        }
    }
    private TreeNode targetNode;

    @Override
    public void doSave(IProgressMonitor monitor) {
    }
    @Override
    public void doSaveAs() {
    }
    private String tabLabel = "";
    /* (non-Javadoc)
     * setter for Editor's prefix in tab label 
     */
    protected void setTabLabel(String value){
        tabLabel = value;
    }
    /* (non-Javadoc)
     * Update a tab name
     */
    protected void updateTabLabel(){
    	UIJob job = new UIJob("update label") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				String partName;
		        if (node instanceof Requirement) {
		            partName = ((Requirement) node).getNameOrId();
		        } else {
		            partName = node.toString();
		        }
		        partName += " ("+node.getType()+")";
		        if(tabLabel.equals("")){
		            setPartName(partName);
		        }else{
		            setPartName(tabLabel+" - "+partName);
		        }
				return Status.OK_STATUS;
			}
		};
		job.schedule();
    }
    
    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        changedNodes = new HashMap<UUID, NodeDesc>();
        changedNodesCount = new HashMap<UUID, Integer>();
        TreeNode sel = null;
        try {
            node = ((NodeEditorInput) input).getStorage().getNode();
            db = node.getTreeDB();
            updateTabLabel();
            sel = ((NodeEditorInput) input).getSelectionStorage().getNode();
        } catch (CoreException e) {
            e.printStackTrace();
        }
        setSite(site);
        setInput(input);
        targetNode = sel;
        TreesTracker.getTracker().addListener(treeStateListener);
        getSite().setSelectionProvider(this);
        //Two handlers to deals with command's exceptions
        IHandlerService handlerService = (IHandlerService) site
                .getService(IHandlerService.class);
        handlerService
            .activateHandler("org.eclipse.ui.edit.copy", new EclipseCopyHandler());
        handlerService
            .activateHandler("org.eclipse.ui.edit.paste", new EclipsePasteHandler());
        //add tree change listener
        db.addListener(treeChangeListener);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    protected String node2jsonString(TreeNode n) throws JSONException {
        String result;
        JSONObject json = NodeMapper.toJSON(n);
        json.put("id", n.getId());
        JSONArray jsonChildren = new JSONArray();
        TreeNode[] children = n.getSortedChildren();
        for (int i = 0; i < children.length; i++) {
            jsonChildren.put(children[i].getUUId().toString());
        }
        json.put("children", jsonChildren);

        JSONObject attr = json.getJSONObject("attributes");
        @SuppressWarnings("rawtypes")
        Iterator keys = attr.keys();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            JSONObject a = attr.getJSONObject(key);
            if (n.getQualifiedId().equals(Requirement.getTypeRootQId())) {
                if (a.getString("key").equals(TreeNode.ATTR_TYPE)) {
                    a.putOpt("value", null);
                }
            }else 
            if (a.getString("key").equals(TreeNode.ATTR_DESCRIPTION)) {
                String descr = n.getDescription(true);
                descr = RequalityCLI.resolveRelativeImagesPath(descr, n);
                a.putOpt("value", descr);
            }else
            if (a.getString("key").equals("_initcond")) {
                String descr = n.getStringAttributeValue("_initcond");
                if(descr!=null){
                	descr = n.processResourcesPath(descr);
                	a.putOpt("value", descr);
                }
            }else
            if (n.getType().equals(TestPurpose.TYPE_NAME) && a.getString("key").equals(TestPurpose.ATTR_EXPECTED_RESULTS)) {
            	a.putOpt("value", ((TestPurpose)n).getExpectedResults(true));
            }
        }
        result = json.toString(4);
        return result;
    }

    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            CommonRemoveLocation.RemoveLocation(loc, req, null, true);
            //TODO: save jsData for undo/redo
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }
    private IPartListener plistener = new IPartListener() {
    	@Override
		public void partOpened(IWorkbenchPart part) {}
		
		@Override
		public void partDeactivated(IWorkbenchPart part) {}
		
		@Override
		public void partClosed(IWorkbenchPart part) {}
		
		@Override
		public void partBroughtToTop(IWorkbenchPart part) {}
		
		@Override
		public void partActivated(IWorkbenchPart part) {
			if(part != JSEditor.this)
				JSEditor.this.browser.execute("saveEditor();");
		}
	};
    @Override
    public void createPartControl(Composite parent) {
        browser = new Browser(parent, RequalityPlugin.BROWSER_TYPE);
        browser.addMouseListener(new MouseListener() {
            @Override
            public void mouseUp(MouseEvent e) {
            }
            
            @Override
            public void mouseDown(MouseEvent e) {
                JSEditor.this.setFocus();
            }
            @Override
            public void mouseDoubleClick(MouseEvent e) {
            }
        });
        browser.addKeyListener(new BrowserKeyListener());

        browser.addProgressListener(new ProgressListener() {
            
            @Override
            public void completed(ProgressEvent event) {
                setSelection(currentSelection);
            }
            
            @Override
            public void changed(ProgressEvent event) {
            }
        });
        getSite().getPage().addPartListener(plistener);
       
        try {
            browser.setUrl(html.toURI().toURL().toString());
            
            new BrowserFunction(browser, "printToConsole") {
                @Override
                public Object function(Object[] arguments) {
                    System.out.println((String)arguments[0]);
                    return null;
                }
            };

            new BrowserFunction(browser, "getNodeByUUID") {
                @Override
                public Object function(Object[] arguments) {
                    UUID nodeUUID = UUID.fromString((String) arguments[0]);
                    TreeNode n = db.getNode(nodeUUID);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };

            new BrowserFunction(browser, "getNodeByQID") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(((String) arguments[0]).replace("&nbsp;", " ").replace((char)160, ' '));
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };
            /*
             * Request for the root element of the selected subtree Input
             * parameters: none Return value: String value containing uuid of
             * the subtree root node
             */
            new BrowserFunction(browser, "getStartNode") {
                @Override
                public Object function(Object[] arguments) {
                    return node.getUUId().toString();
                }
            };
            
            new BrowserFunction(browser, "getSelectedNode") {
                @Override
                public Object function(Object[] arguments) {
                    if(currentSelection == null)
                        return node.getUUId().toString();
                    return currentSelection.getUUId().toString();
                }
            };

            /*
             * Request to change the node ID Input parameters: uuid - (String)
             * uuid of the updated node new_id - (String) new ID to be set
             * Return value: Boolean value: true if operation was successful,
             * false otherwise
             */
            new BrowserFunction(browser, "changeNodeId") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String newId = (String) arguments[1];
                    if (newId.equals("")) {
                        refresh(n);
                    } else {
                        try {
                            newId = newId.replace((char)160, ' ').replace("&nbsp;", " ").trim();
                            if (db.getTreeLogic().isIdValidExc(n, newId)){
                                db.startTransaction("Update " + splitWords(n.getType()) + " Id");
                                n.setId(newId);
                                db.commit();
                            }else{
                                MessageDialog.openError(PlatformUI.getWorkbench()
                                        .getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                                        "Id of element cannot be changed to " + newId);
                                return false;
                            }
                        } catch (SameIdExistsInChildren e){
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                                    "Element with same id exists in " + 
                                    n.getParent().getUserFriendlyName());
                            return false;
                        } catch (IdEmpty e) {
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                                    "Element id cannot be empty");
                            return false;
                        } catch (IdContainsWrongChars e) {
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                                    "Element id cannot contains wrong character - " 
                                    + e.getWrongCharacter());
                            return false;
                        }
                    }
                    return true;
                }
            };

            /*
             * Wrapping method to call Java dialog for id altering
             * First argument is UUID of parent to support
             * id validation. 
             * Next three arguments are title of shown dialog, message
             * and default value for dialog.
             * Return value: String value: null if dialog was Canceled,
             * valid id otherwise
             */
            new BrowserFunction(browser, "openIdChange") {
                @Override
                public Object function(Object[] arguments) {
                    final TreeNode nde = db.getNode(UUID.fromString((String) arguments[0]));
                    String title = (String)arguments[1],
                            message = (String)arguments[2],
                            lastValue = (String)arguments[3];
                    boolean idcheck = true;
                    if(arguments.length>4)
                        idcheck = (Boolean)arguments[4];
                    final boolean checkId = idcheck;
                    InputDialog id = new InputDialog(this.getBrowser().getShell(), title, message
                            ,lastValue ,new IInputValidator() {
                                @Override
                                public String isValid(String newText) {
                                    if (newText.length() == 0)
                                        return "id cannot be empty";
                                    if(checkId){
                                        if(!nde.getTreeDB().getTreeLogic().isChildIdValid(nde, newText))
                                            return newText + " is not valid id for the child of " + nde.getUserFriendlyName()+"("+nde.getType()+")";
                                    }
                                    return null;
                                }
                            });
                    id.open();
                    browser.forceFocus();
                    if(id.getReturnCode() == InputDialog.CANCEL)
                        return null;
                    return id.getValue();
                }
            };
            new BrowserFunction(browser, "getNextAvalibleId") {
                @Override
                public Object function(Object[] arguments) {
                    final TreeNode nde = db.getNode(UUID.fromString((String) arguments[0]));
                    return RequalityCLI.getNextId(nde, nde.getType(), null);
                }
            };
            new BrowserFunction(browser, "changeNodeName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    if (!n.getType().equals(Requirement.TYPE_NAME)) {
                        return false;
                    }
                    String newName = (String) arguments[1];
                    if(newName != null)
                        newName = newName.replace((char)160, ' ').replace("&nbsp;", " ").trim();
                    db.startTransaction("Change " + splitWords(n.getType()) + " Name");
                    ((Requirement) n).setName(newName);
                    n.saveAttributes();
                    db.commit();
                    return true;
                }
            };

            new BrowserFunction(browser, "getUFName") {
                @Override
                public Object function(Object[] arguments) {
                    try{
                        TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                        return n.getUserFriendlyName().replace("  ", "&nbsp;&nbsp;");
                    }
                    catch(RuntimeException re){
                        return null;
                    }
                }
            };
            /*
             * Returns true if it is possible to edit this node
             */
            new BrowserFunction(browser, "canEditNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    return db.getTreeLogic().canBeDeleted(n);
                }
            };
            /*
             * Check the possibility of adding some node as
             *  child of another. Also can be used to check
             *  posibily of adding a child with some name
             * 
             */
            new BrowserFunction(browser, "checkName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = null;
                    String newName = (String) arguments[1];
                    if(newName != "")
                        newName = newName.replace((char)160, ' ').replace("&nbsp;", " ").trim();
                    if(!((String)arguments[0]).equals("")){
                        n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                        if(n.getType().equals(Requirement.TYPE_NAME)){
                            Attribute a = n.getAttribute(Requirement.ATTR_NAME);
                            if(a != null && a.getValue() != null && a.getValue() != "")
                                return true;
                        }
                        if(newName.equals(""))
                            newName = n.getId();
                    }
                    TreeNode parent = null;
                    if(n != null)
                        parent = n.getParent();
                    if(arguments.length > 2){
                        parent = db.getNode(UUID
                                .fromString((String) arguments[2]));
                    }
                    //TODO maybe change to string error message
                    if((n == null) && (parent == null) || newName.equals(""))
                        return false;
                    if(n != null && n.getType().equals(Comment.TYPE_NAME))
                        return true;
                    return reqIdValid(n,parent,newName);
                }
            };
            /*
             * Check if it possible to add some node 
             * as child of another one
             * 
             */
            new BrowserFunction(browser, "checkChild") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = null;
                    String newName = ((String) arguments[1]).replace("&nbsp;", " ").replace((char)160, ' ').trim();
                    if(!((String)arguments[0]).equals("")){
                        n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                        if(newName.equals(""))
                            newName = n.getId();
                    }
                    TreeNode parent = null;
                    if(arguments.length > 2){
                        parent = db.getNode(UUID
                                .fromString((String) arguments[2]));
                    }
                    if((n == null) && (parent == null) || newName.equals(""))
                        return false;
                    if(n != null && n.getType().equals(Comment.TYPE_NAME))
                        return true;
                    return reqChildValid(n, parent, newName);
                }
            };
            
            /*
             * Request to change the value of one of the node attributes Input
             * parameters: uuid - (String) uuid of the updated node attr_name -
             * (String) name of the changed attribute new_val - (String) new
             * attribute value to be set Return value: Boolean value: true if
             * operation was successful, false otherwise
             */
            new BrowserFunction(browser, "changeNodeAttr") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        UUID uuid = UUID.fromString((String) arguments[0]);
                        TreeNode n = db.getNode(uuid);
                        String key = (String) arguments[1];
                        String value = (String) arguments[2];
                        synchronized (JSEditor.this) {
                        	if(n.getType().equals(Requirement.TYPE_NAME) && key.equals(Requirement.ATTR_NAME)){
                        		if (((Requirement)n).getName() != null) {
                        			if(value != null)
                        				value = value.replace((char)160, ' ').trim();
                        			if(value.charAt(0) == ((char)255)){
                        				refresh(n);
                        			}else{
                        				db.startTransaction("Change Requirement Name");
                        				((Requirement)n).setName(value);
                        				n.saveAttributes();
                        				db.commit();
                        			}
                        		} else
                        			if (((Requirement)n).getName() == null) {
                        				if(value != null)
                        					value = value.replace((char)160, ' ').replace("&nbsp;"," ").trim();
                        				if (value.charAt(0) == ((char)255)) {
                        					refresh(n);
                        				} else {
                        					try{
                        						if (db.getTreeLogic().isIdValidExc(n, value)){
                        							db.startTransaction("Update " + splitWords(n.getType()) + " Id");
                        							((Requirement)n).setId(value);
                        							db.commit();
                        							Activator.getDefault().setActiveNode(n, n.getType());
                        						}else{
                        							MessageDialog.openError(PlatformUI.getWorkbench()
                        									.getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                        									"Id of element cannot be changed to " + value);
                        							return false;
                        						}
                        					}
                        					catch(SameIdExistsInChildren e){
                        						MessageDialog.openError(PlatformUI.getWorkbench()
                        								.getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                        								"Element with same id exists in " + 
                        										n.getParent().getUserFriendlyName());
                        						return false;
                        					}
                        					catch(IdEmpty e){
                        						MessageDialog.openError(PlatformUI.getWorkbench()
                        								.getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                        								"Element id cannot be empty");
                        						return false;
                        					}
                        					catch(IdContainsWrongChars e){
                        						MessageDialog.openError(PlatformUI.getWorkbench()
                        								.getActiveWorkbenchWindow().getShell(), "Update " + splitWords(n.getType()) + " Id",
                        								"Element id cannot contains wrong character - " 
                        										+ e.getWrongCharacter());
                        						return false;
                        					}
                        				}
                                }
                            } else {
                                db.startTransaction("Update " + splitWords(n.getType()));
                                if (key.equals(TreeNode.ATTR_DESCRIPTION) || key.equals(TestPurpose.ATTR_EXPECTED_RESULTS) || key.equals("_initcond")) {
                                    RequalityCLI.deleteOldImages(n.getAttribute(key), value, n);
                                    value = RequalityCLI.copyImages(value, n).replace((char)160, ' ');
                                    value = value.replace("\"" + n.getResourcesPath(), "\"${node.resURL}");
                                }
                                Attribute attr = null;
                                if (!value.equals(""))
                                {
                                    attr = new Attribute(AttributeType.STRING, key, value);
                                    n.putAttribute(attr);
                                } else {
                                    n.removeAttribute(key);
                                }
                                if (!changedNodes.containsKey(uuid)) {
                                    changedNodes.put(uuid, n.toNodeDesc());
                                    changedNodesCount.put(uuid, 1);
                                } else {
                                    NodeDesc nodeDesc = changedNodes.get(uuid);
                                    if (attr != null) {
                                        nodeDesc.setAttribute(attr);
                                    } else {
                                        nodeDesc.removeAttribute(key);
                                    }
                                    changedNodesCount.put(uuid, changedNodesCount.get(uuid) + 1);
                                }
                                n.saveAttributes();
                                db.commit();
                            }
                        }
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                    return true;
                }
            };

            /*
             * Request to change the move a node into different subtree Input
             * parameters: uuid - (String) uuid of the moved node
             * new_parent_uuid - (String) uuid of the new parent node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "changeNodeParent") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[1]));
                    Index i = null;
                    TreeNode lastNode = null;
                    TreeLogic l = parent.getTreeDB().getTreeLogic();
                        if(!l.canBeChildOf(n.getType(), parent) &&
                                l.canBeChildOf(n.getType(), parent.getParent())&&
                                l.isSiblinkPasteAvalible(n.getType())){
                            lastNode = parent;
                            parent = parent.getParent();
                        }
                    //if we deal with test step
                    //then special actions performed
                    if( n.getAttribute("_index") != null)
                        i = Index.getNextIndex(n.getType(), parent, lastNode);
                    try {
                        //if(!l.canBeChildOf(n.getType(), parent))
                          //  parent = parent.getParent();
                    if (!n.getParent().equals(parent) || (i != null)) {
                        if(!l.canBeCopiedToExc(n, parent, n.getType().equals(Comment.TYPE_NAME))){
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                    "Element " + n.getUserFriendlyName() + " parent cannot be changed to "
                                    + parent.getUserFriendlyName());
                            return false;
                        }
                        if(n == null || parent == null)
                            return false;
                        db.startTransaction("Move Element");
                        if(i != null){
                            n.setIndex(i);
                            n.saveAttributes();
                        }
                        if(!n.getParent().equals(parent) && l.canBeChildOf(n.getType(), parent)){
                            String torename = null;
                            if(l.isAutoRenamable(n.getType()) || parent.findChild(n.getId()) != null){
                                if(n.getType().equals(Comment.TYPE_NAME)){
                                    torename = RequalityCLI.getNextId(parent, n.getType(), Comment.TYPE_NAME + " ");
                                }else
                                    torename = RequalityCLI.getNextId(parent, n.getType(), null);
                                n.setId(RequalityCLI.getNextId(parent, n.getParent(), n.getType(), null));
                            }
                            n.setParent(parent);
                            if(torename != null)
                                n.setId(torename);
                        }
                        db.commit();
                        setSelection(n);
                        refresh(parent);
                        return true;
                    }
                    } catch (PastingIntoLeafException e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                "Can't change parent of " + n.getType() + " to "
                                + parent.getType() + " with "
                                + (n.getType().equals(TestPurpose.TYPE_NAME) ? "non-" :"")
                                + "TestPurpose child");
                        return false;
                    } catch (WrongProjectException e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                "Moving elements between projects is prohibited");
                        return false;
                    } catch (SameIdExistsInChildren e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                "Element with same id exists in " + 
                                parent.getUserFriendlyName());
                        return false;
                    } catch (IdEmpty e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                "Element id is empty");
                        return false;
                    } catch (IdContainsWrongChars e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Change Element parent",
                                "Element id contains wrong character - " 
                                + e.getWrongCharacter());
                        return false;
                    }
                    return false;
                }
            };

            /*
             * Request to insert a new node Input parameters: uuid - (String)
             * uuid of the new node parent_uuid - (String) uuid of the parent
             * node Return value: Boolean value: true if operation was
             * successful, false otherwise
             */
            new BrowserFunction(browser, "addNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String id = (String) arguments[1];
                    String type = (String) arguments[2];
                    TreeLogic l = db.getTreeLogic();
                    if (id == null || id.length() == 0) {
                        if (type.equals(Comment.TYPE_NAME)) {
                            id = RequalityCLI.getNextId(parent, type, Comment.TYPE_NAME + " ");
                        } else {
                            if(type.equals(TestPurpose.TYPE_NAME)){
                                String parid = parent.getId();
                                if(parent instanceof Requirement)
                                    parid = ((Requirement)parent).getNameOrId();
                                id = RequalityCLI.getNextId(parent, type, parid+"_T");
                            }else{
                                id = RequalityCLI.getNextId(parent, type, null);
                            }
                        }
                    }
                    id = id.replace((char)160, ' ').replace("&nbsp;", " ").trim();
                    try {
                        if(!l.canBeCreatedInExc(type, id, parent)){
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                                    "Element of type '" + type 
                                    + "' cannot be created in " + parent.getUserFriendlyName());
                            return null;
                        }
                    } catch (PastingIntoLeafException e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Can't add "+type+" to element with " + (type.equals(TestPurpose.TYPE_NAME)?"non-":"")
                                + "TestPurpose child");
                        return null;
                    } catch (SameIdExistsInChildren e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Node",
                                "Element with same id exists in " + 
                                parent.getUserFriendlyName());
                        return null;
                    } catch (IdEmpty e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Node",
                                "Element id is empty");
                        return null;
                    } catch (IdContainsWrongChars e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Node",
                                "Element id contains wrong character - " 
                                + e.getWrongCharacter());
                        return null;
                    }
                    db.startTransaction("Add Element");
                    Index ind = null;
                    if(type.equals(Comment.TYPE_NAME)){
                        if( arguments.length < 4 || arguments[3] == null || ((String)arguments[3]).length() == 0 )
                        {
                            ind = (Index.getNextIndex(type, parent, null, false));
                        }else{
                            ind = (Index.getNextIndex(type, parent, db.getNode(UUID
                                    .fromString((String) arguments[3])), false));
                        }
                    }
                    TreeNode newNode = parent.createChild(id, type);
                    //TODO: remake to commands
                    if (type.equals(TestPurpose.TYPE_NAME)) {
                        newNode.putAttribute(new Attribute(AttributeType.STRING,
                                TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
                        newNode.putAttribute(new Attribute(AttributeType.STRING,
                                TestPurpose.ATTR_STATUS, "in process"));
                        newNode.saveAttributes();
                    }
                    //for comments we adds default author
                    if (type.equals(Comment.TYPE_NAME)){
                        newNode.putAttribute(new Attribute(AttributeType.STRING,
                                TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
                        newNode.setIndex(ind);
                        newNode.saveAttributes();
                    }
                    db.commit();
                    return newNode.getUUId().toString();
                }
            };

            /*
             * Request to remove a node with all its decendants Input
             * parameters: uuid - (String) uuid of the removed node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "removeNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    List<TreeNode>nds= new ArrayList<TreeNode>();
                    nds.add(n);
                    try {
						DeleteSelection.deleteNodes(nds);
						return true;
					} catch (ExecutionException e) {
						e.printStackTrace();
					}
                    //if(!db.getTreeLogic().canBeDeleted(n))
                        //return false;
                    //db.startTransaction("Remove Element");
                    //if (n instanceof Requirement) {
                        //deleteLocations((Requirement)n);
                    //}
                    //n.delete();
                    //db.commit();
                    return false;
                }
            };
            new BrowserFunction(browser, "setRequalityFocus") {
                @Override
                public Object function(Object[] arguments) {
                    // workaround (ignore first unieditor selection change)
                    if(currentSelection == null) {
                        currentSelection = targetNode!=null?targetNode:db.getNode(UUID
                                .fromString((String) arguments[0]));
                    } else {
                        currentSelection = db.getNode(UUID
                                .fromString((String) arguments[0]));
                    }
                    if (uniEditorOutlinePage != null) {
                        uniEditorOutlinePage
                                .setSelection(new StructuredSelection(
                                        currentSelection));
                    }
                    // synchronization with Requality Explorer
                    IViewPart view = PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getActivePage()
                            .findView(RequalityCNF.ID);
                    if (view instanceof CommonNavigator) {
                        CommonNavigator navigator = (CommonNavigator) view;
                        if (navigator.isLinkingEnabled()) {
                            navigator.setLinkingEnabled(true);
                        }
                    }
                    fireSelectionChange();
                    return true;
                }
            };

            /**
             * returns absolute URL to project folder
             */
            new BrowserFunction(browser, "getBaseUrl") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        return RequalityPlugin.getTreesTracker().getProject(db)
                                .getLocation().toFile().toURI().toURL()
                                .toString();
                    } catch (MalformedURLException e) {
                        e.printStackTrace();
                    }
                    return null;
                }
            };
            

            new BrowserFunction(browser, "getReadOnlyMode") {
                @Override
                public Object function(Object[] arguments) {
                    return editMode;
                }
            };

            
            new BrowserFunction(browser, "copyNode") {
                @Override
                public Object function(Object[] arguments) {
                    try{
                        TreeNode node = db.getNode( UUID.fromString( (String)arguments[0] ) );
                        if(!node.getTreeDB().getTreeLogic().isCopyable(node))
                            return false;
                        boolean cutNode = false;
                        if(arguments.length>1)
                            cutNode = (Boolean)arguments[1];
                        (new CopyHandler()).copyNodes(new TreeNode[]{node},cutNode);
                    }
                    catch(Exception e){
                        return false;
                    }
                    return true;
                }
            };

            new BrowserFunction(browser, "pasteNode") {
                @Override
                public Object function(Object[] arguments) {
                    try{
                        TreeNode parent = db.getNode( UUID.fromString( (String)arguments[0] ) );
                        Index newIndex = null;
                        boolean pasteTree = false;
                        if(arguments.length>1)
                            pasteTree = (Boolean)arguments[1];
                        TreeNode ret = (TreeNode) (new PasteIntoHandler()).pasteNodes(parent, pasteTree);
                        //test step, temporary here
                    }
                    catch(RuntimeException e){
                        return false;
                    }
                    return true;
                }
            };
            /**
             * Method to get all possible nodes with given User Visible Name
             * first argument - User Visible Name
             * returns Array of Strings (UUIDs)
             */
            new BrowserFunction(browser, "getNodesByUFName") {
                @Override
                public Object function( Object[] arguments ) {
                    List<String> filter = db.getLinkToTypeFilter();
                    if (arguments.length > 2){
                        filter = new ArrayList<String> ();
                        String[]tp = ((String)arguments[2]).split(",");
                        for(String s : tp)
                            if(!s.equals(""))
                                filter.add(s);
                    }
                    String [] nd = db.getNodesByUFName(((String)arguments[0]).replace((char)160, ' ').replace("&nbsp;", " ").trim(), filter);
                    if(nd == null)
                        return null;
                    Object [] nodes = new Object[nd.length];
                    for(int i = 0; i < nd.length; i++)
                        nodes[i] = nd[i];
                    return nodes;
                }
            };
            /**
             * Show dialog of selection from list of UUIDs. Then
             * ask for selection.
             * argument 1 - Array of UUID in form of strings
             * returns selected nodes or null
             */
            new BrowserFunction(browser, "fireNodeSelectionFromList") {
                @Override
                public Object function( Object[] arguments ) {
                    List<TreeNode>nodes = new ArrayList<TreeNode>();
                    Object[]o = (Object[])arguments[0];
                    for(int i = 0; i < o.length; i++){
                        //check if node exists
                        TreeNode node = db.getNode(UUID.fromString((String)o[i]));
                        if(node != null)
                            nodes.add(node);
                    }
                    String caption = "";
                    //if we have 2 arguments, then second one
                    //is caption of dialog
                    if(arguments.length > 1){
                        caption = (String)arguments[1];
                    }
                    if(nodes.size() == 0)
                        return null;
                    SelectNodeDialog selDialog = new SelectNodesFromList(
                            nodes, getSite().getShell(), db, caption);
                    if (selDialog.open() == SelectNodeDialog.OK) {
                        TreeNode res = selDialog.getResult();
                        return res.getUUId().toString();
                    }
                    return null;
                }
            };

            new BrowserFunction(browser, "getLastChildUUID") {
                @Override
                public Object function( Object[] arguments ) {
                    TreeNode parent = db.getNode( UUID.fromString( (String)arguments[0] ) );
                    TreeNode[] children = parent.getSortedChildren();
                    if( children == null || children.length == 0 ) return null;
                    return children[children.length-1].getUUId().toString();
                }
            };

            // [0] - test procedure UUID
            // [1] - previous test step UUID
            new BrowserFunction( browser, "createTestStep" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode parent = db.getNode( UUID.fromString((String) arguments[0]));
                    try {
                        if(!db.getTreeLogic().canBeCreatedInExc("TestStep", parent) ){
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                                    "Element of type Test Step" 
                                    + " cannot be created in " + parent.getUserFriendlyName());
                            return null;
                        }
                    } catch (PastingIntoLeafException e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Can't add Test Step to "
                                + parent.getType() + " with "
                                + (node.getType().equals(TestPurpose.TYPE_NAME) ? "non-" : "")
                                + "Test Purpose child");
                        return null;
                    }
                    if( arguments.length < 2 || arguments[1] == null || ((String)arguments[1]).length() == 0 )
                    {
                        // no children
                        createTestStep( parent, Index.createOne() ); 
                        return null;
                    }
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[1]));
                    TreeNode ts = createTestStep(parent, Index.getNextIndex("TestStep", parent, node));
                    return ts.getUUId().toString();
                }
            };
            /**
             * Method to remove node, that key(UUID or "*"+UUID)
             * starts with argument 2
             * argument 1 - UUID of target node
             * returns true or null
             */
            new BrowserFunction( browser, "removeLink" ){
                @SuppressWarnings("unchecked")
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[0]));
                    if(node == null)
                        return null;
                    Attribute a = node.getAttribute(TreeNode.ATTR_LINKS);
                    if(a == null)
                        return null;
                    List<String> l = (List<String>) a.getValue();
                    if(arguments[1]== null || ((String)arguments[1]).length()<1)
                        return null;
                    for(String s:l)
                        if(s.substring(0,s.indexOf('_')).startsWith((String)arguments[1])){
                            l.remove(s);
                            node.getTreeDB().startTransaction("remove single Link");
                            node.putAttribute(new Attribute(AttributeType.LIST,TreeNode.ATTR_LINKS,l));
                            node.saveAttributes();
                            node.getTreeDB().commit();
                            return true;
                        }
                    
                    return null;
                }
            };
            /**
             * Returns UUId of new test purpose, selected by dialog 
             * returns UUId in form of string or null
             */
            new BrowserFunction( browser, "addNewTestPurposeToEditor" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    //TreeNode node = db.getNode( UUID.fromString((String) arguments[0]));
                    SelectNodeDialog selDialog = new SelectTestPurposeDialog(
                            getSite().getShell(), db);
                    if (selDialog.open() == SelectNodeDialog.OK) {
                        TreeNode res = selDialog.getResult();
                        return res.getUUId().toString();
                    }
                    return null;
                }
            };
            
            new BrowserFunction( browser, "canContainType" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[0]));
                    return node.getTreeDB().getTreeLogic().canBeChildOf((String)arguments[1], node);
                }
            };
            new BrowserFunction( browser, "canBeChildTypeOf" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    return db.getTreeLogic().canBeChildOf((String) arguments[0], (String)arguments[1]);
                }
            };
            /** 
             * Shows dialog and then ask for choose or add. 
             * returns absolute URL to project folder
             */
            new BrowserFunction( browser, "addSingleTestPurposeToTestStep" )
            {
                @SuppressWarnings("unchecked")
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[0]));
                    SelectNodeDialog selDialog = new SelectTestPurposeDialog(
                            getSite().getShell(), db);
                    if (selDialog.open() == SelectNodeDialog.OK) {
                        TreeNode res = selDialog.getResult();
                        //TODO ss
                        ArrayList<String> al = (ArrayList<String>) node.getAttributeValue(TreeNode.ATTR_LINKS);
                        if(al == null)
                            al = new ArrayList<String>();
                        Iterator<String>i = al.iterator();
                        while(i.hasNext()){
                            String str = i.next();
                            //if we already have hidden link
                            //then remove it
                            if(str.startsWith("*"))
                                i.remove();
                        }
                        al.add("*" + res.getUUId().toString() +
                                "_" + res.getUserFriendlyName());
                        node.getTreeDB().startTransaction("Add Single Test Purpose");
                        node.putAttribute(new Attribute(AttributeType.LIST,TreeNode.ATTR_LINKS,al));
                        node.saveAttributes();
                        node.getTreeDB().commit();
                        refresh(node);
                        refresh(res);
                        return res.getUUId().toString();
                    }
                    return null;
                }
            };
            /**
             * Open Editor for node. If it is not possible,
             * then open Editor to node's parent. Otherwise,
             * do nothing
             */
            new BrowserFunction( browser, "openEditorForNode" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    final TreeNode nde = db.getNode( UUID.fromString((String) arguments[0]));
                    TreeNode node = nde;
                    final TreeNode sel = nde;
                            String tp = node.getType();
                            if(tp.equals(TestPurpose.TYPE_NAME)
                                    ||tp.equals(Comment.TYPE_NAME)
                                    ||tp.equals("TestStep")
                                    ||tp.equals(Location.TYPE_NAME))
                                node = node.getParent();
                            NodeEditorInput input = new NodeEditorInput(node, sel);
                            String editorid = UniEditor.ID;
                            try 
                            {
                                if(tp.equals(Document.TYPE_NAME))
                                    editorid = ReqMarker.ID;
                                IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), input, editorid, false);
                            } 
                            catch(PartInitException e) 
                            {
                                e.printStackTrace();
                            }
                    return null;
                }
            };
            
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    @Override
    public void setFocus() {
        new UndoRedoActionGroup(getSite(), PlatformUI.getWorkbench()
                .getOperationSupport().getUndoContext(), true)
                .fillActionBars(getEditorSite().getActionBars());
        browser.setFocus();
        fireSelectionChange();
    }
    public void dispose() {
    	getSite().getPage().removePartListener(plistener);
        db.removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    public JSEditor() {
        super();
        try {
            html = new File(Activator.getPath("jstools/unieditor.html"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected void refresh(final TreeNode node) {
        // Refresh nothing but root node subtree
        String rootQID = this.node.getQualifiedId();
        String nodeQID = node.getQualifiedId();
        
        if (!nodeQID.startsWith(rootQID + "/") && (!nodeQID.equals(rootQID))) {
            if(node.getType().equals("TestPurpose")&&(this.node.getType().equals("TestProcedure")))
                browser.execute("testStepUpdate('" + node.getUUId()
                        + "');");
            return;
        }
        if(rootQID.equals(nodeQID))
            updateTabLabel();
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    synchronized (JSEditor.this) {
                        if (changedNodes.containsKey(node.getUUId())) {
                            NodeDesc nd = changedNodes.get(node.getUUId());
                            if (nd.equals(node.toNodeDesc())) {
                                Integer count = changedNodesCount.get(node
                                        .getUUId());
                                if (count > 0) {
                                    changedNodesCount.put(node.getUUId(),
                                            changedNodesCount.get(node
                                                    .getUUId()) - 1);
                                    return;
                                }
                            }
                            changedNodes.remove(node.getUUId());
                            changedNodesCount.remove(node.getUUId());
                        }
                        browser.execute("onChangeNode('" + node.getUUId()
                                + "');");
                        browser.forceFocus();
                    }
                }
            }
        });
    }

    protected void setUniEditorFocus(final UUID uuId) {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    browser.execute("onChangeFocus('" + uuId + "');");
                }
            }
        });
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Object getAdapter(Class required) {
        if (IContentOutlinePage.class.equals(required)) {
            if (uniEditorOutlinePage == null) {
                uniEditorOutlinePage = new UniEditorOutlinePage();
                uniEditorOutlinePage
                        .addSelectionChangedListener(outlineSelectionChangeListener);
                uniEditorOutlinePage.setInput(this.node);
            }
            return uniEditorOutlinePage;
        }
        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(new ITabbedPropertySheetPageContributor() {
                @Override
                public String getContributorId() {
                    return "com.unitesk.requality.views.RequalityPropertyContributor";
                }
            });
        }
        if(required.equals(IContributedContentsView.class)){
        	return new IContributedContentsView() {
                public IWorkbenchPart getContributingPart() {
                    return PlatformUI
                            .getWorkbench()
                            .getActiveWorkbenchWindow()
                            .getActivePage()
                            .findView(
                                    "com.unitesk.requality.views.RequalityExplorer");
                }
            };
        }
        return super.getAdapter(required);
    }

    public Browser getBrowser() {
        return browser;
    }

    public void setEditMode(int editMode) {
        this.editMode = editMode;
    }

    private void closeEditor() {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                getEditorSite().getPage()
                        .closeEditor(JSEditor.this, false);
            }
        });
    }

    // //////////////////////////////////////////////////////////////////////////
    //
    // Selection provider
    //
    // //////////////////////////////////////////////////////////////////////////

    private List<ISelectionChangedListener> selectionChangeListeners = new ArrayList<ISelectionChangedListener>();

    @Override
    public void addSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.add(listener);
    }

    @Override
    public void removeSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.remove(listener);
    }

    @Override
    public ISelection getSelection() {
        if (currentSelection == null) {
            return StructuredSelection.EMPTY;
        }
        return new StructuredSelection(currentSelection);
    }

    @Override
    public void setSelection(ISelection selection) {
    }

    private void fireSelectionChange() {
        ISelection sel = this.getSelection();
        SelectionChangedEvent event = new SelectionChangedEvent(this, sel);
        for (ISelectionChangedListener listener : selectionChangeListeners) {
            listener.selectionChanged(event);
        }
    }

    private TreeNode createTestStep( TreeNode parent, Index index )
    {
        String type = "TestStep";
        String id = RequalityCLI.getNextId(parent, type, null);
        if (parent.findChild(id) == null) {
            try {
                if(!db.getTreeLogic().canBeCreatedInExc(type, id, parent)){
                    MessageDialog.openError(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getShell(), "Add Element",
                            "Element of type 'TestStep"
                            + "' cannot be created in " + parent.getUserFriendlyName());
                    return null;
                }
            } catch (PastingIntoLeafException e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Can't add " + splitWords(node.getType()) + " to "
                        + splitWords(parent.getType()) + " with "
                        + (node.getType().equals(TestPurpose.TYPE_NAME) ? "non-" : "")
                        + "Test Purpose child");
                return null;
            } catch (SameIdExistsInChildren e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Element with same id exists in " + 
                        parent.getUserFriendlyName());
                return null;
            } catch (IdEmpty e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Element id is empty");
                return null;
            } catch (IdContainsWrongChars e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Add Element",
                        "Element id contains wrong character - " 
                        + e.getWrongCharacter());
                return null;
            }
            db.startTransaction("Add Test Step");
            TreeNode newNode = parent.createChild( id, type );
            newNode.setIndex( index );
            newNode.saveAttributes();
            db.commit();
            return newNode;
        }
        return null;
    }

    private String splitWords(String s) {
        if (s == null)
            return null;
        for (int i = 1; i < s.length(); i++) //s.length() is variable
            if (s.charAt(i - 1) >= 'a' && s.charAt(i - 1) <= 'z' &&
                s.charAt(i) >= 'A' && s.charAt(i) <= 'Z') {
                s = s.substring(0, i) + " " + s.substring(i);
            }
        return s;
    }

}
