package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.EditorPart;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipseCopyHandler;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipsePasteHandler;
import com.unitesk.requality.eclipse.handlers.report.OpenReportHandler;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.report.Template;

// TODO: refacatoring
public class ReportViewer extends EditorPart {
    public static String ID = "com.unitesk.requality.editors.browser.ReportViewer";
    private ReportData rdata;
    private Browser browser;
    private String href;

    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void delete(IProject proj) {
            if (TreesTracker.getResourceByNode(rdata).getProject().equals(proj)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }

        @Override
        public void close(TreeDB db) {
            if (rdata.getTreeDB().equals(db)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }
    };

    INodeChangeListener nodeChangeListener = new INodeChangeListener() {
		
		@Override
		public void deleted(TreeNode source) {
			if(source.getUUId().equals(rdata.getUUId()))
				getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
		}
		
		@Override
		public void attributeChange(TreeNode node, String what) {
			if(what.equals(ReportData.ATTR_TEMPLATE) || 
					what.equals(TreeNode.ATTR_UNDEFINED)){
				if(! rdata.getQualifiedId().equals(node.getQualifiedId())){
	                return;
	            }
	            Template tpl = Activator.getDefault().getReportTemplates()
	                    .get(rdata.getTemplateId());
	            String path = null;
	            if (tpl == null) {
	                // get first
	            	if(rdata.getTemplateId() != null)
	            		MessageDialog.openError(PlatformUI.getWorkbench()
	                        .getActiveWorkbenchWindow().getShell(), "Report generation",
	                        "Report template '" + rdata.getTemplateId() + "' for this report is not avalible.");
	                path = ((ReportData)node).getSaveDir() + java.io.File.separator +
	                    (String)node.getAttributeValue(ReportData.ATTR_LAST_GENERATED);
	            } else {
	                path = tpl.getTemplates().get(0)
	                        .getOutputFileName();
	            }
	            final IResource res = OpenReportHandler.getReportFolder((ReportData) node).getFile(
	                    Path.fromOSString(path));
	            if (res.exists()) {
	                getSite().getShell().getDisplay().syncExec(new Runnable() {
	                    @Override
	                    public void run() {
	                        OpenReportHandler.openReport(rdata);
	                        if(!browser.isDisposed())
	                        	browser.refresh();
	                    }
	                });
	            } else {
	                getSite().getShell().getDisplay().syncExec(new Runnable() {
	                    @Override
	                    public void run() {
	                        getEditorSite().getPage().closeEditor(
	                                ReportViewer.this, false);
	                    }
	                });
	            }
			}
		}
	};
    
    private String reportName;

    public ReportViewer() {
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        setSite(site);
        setInput(input);
        try {
            rdata = (ReportData) ((NodeEditorInput) input).getStorage()
                    .getNode();
            reportName = rdata.getId();
            Template tpl = Activator.getDefault().getReportTemplates()
                    .get(rdata.getTemplateId());
            String savePath = rdata.getSaveDir();
            if (tpl == null) {
                if(rdata.getTemplateId() != null){
                    String lastGen = (String) rdata.getAttributeValue(ReportData.ATTR_LAST_GENERATED);
                    if(lastGen != null){
                        href = Activator.getUrl(savePath + lastGen).toString();
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Report generation",
                                "Report template '" + rdata.getTemplateId() + "' for this report is not avalible.");
                    }//else we have new template and it is ridiculous
                }else{
                    tpl = Activator.getDefault().getReportTemplates().values()
                                  .iterator().next();
                }
            }
            if (tpl != null)
                href = Activator.getUrl(savePath + File.separator
                        + tpl.getTemplates().get(0).getOutputFileName()).toString();
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }
 
    @Override
    public void createPartControl(Composite parent) {
        setPartName("Report: " + reportName);
        browser = new Browser(parent, RequalityPlugin.BROWSER_TYPE);
        IHandlerService handlerService = (IHandlerService) getSite()
                .getService(IHandlerService.class);
        handlerService
            .activateHandler("org.eclipse.ui.edit.copy", new EclipseCopyHandler());
        handlerService
            .activateHandler("org.eclipse.ui.edit.paste", new EclipsePasteHandler());
        
        if (href != null){
            browser.setUrl(href);
            rdata.addChangeListener(nodeChangeListener);
            TreesTracker.getTracker().addListener(treeStateListener);
        }
    }

    @Override
    public void dispose() {
    	rdata.removeChangeListener(nodeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    @Override
    public void setFocus() {
        browser.setFocus();
    }

}
