package com.unitesk.requality.eclipse.views;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.SortedMap;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.events.HyperlinkEvent;
import org.eclipse.ui.forms.events.IHyperlinkListener;
import org.eclipse.ui.forms.widgets.Hyperlink;

import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;

public class RequalityPreferences extends PreferencePage implements
        IWorkbenchPreferencePage {
    public static String BROWSER_ENGINE = "browser_engine";
    private Combo list;
    private Label activeEngine;
    private SortedMap<Integer, Boolean> engState;
    private Map<Integer,Integer> mappingIndex = 
            new HashMap<Integer, Integer>();
    //text on top of preference page
    private static String desciption_text = 
            "Requality requires browser engine to be integrated to Eclipse." +
    		"\nForm below shows settings on possible engines." +
            "\nWarning: changes on this page causes restart of eclipse." +
    		"\nMore information about engines can be found in official manual:";
    //information about link to manual
    private static String link_text = "Manual";
    private static String link_href = "/com.unitesk.requality/docs/manual/general.en.html";
    public RequalityPreferences() {
    }

    public RequalityPreferences(String title) {
        super(title);
    }

    public RequalityPreferences(String title, ImageDescriptor image) {
        super(title, image);
        // TODO Auto-generated constructor stub
    }

    @Override
    public void init(IWorkbench workbench) {
    }
    @Override
    protected Control createContents(Composite parent) {
        //panel for texts
        Composite desccompose = new Composite(parent, SWT.FILL);
        desccompose.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        desccompose.setLayout(new GridLayout());
        Label descl = new Label(desccompose, SWT.HORIZONTAL);
        descl.setText(desciption_text);
        //Hyperlink to manual
        final Hyperlink desc = new Hyperlink(desccompose, SWT.NONE);
        desc.setHref(link_href);
        desc.setUnderlined(true);
        desc.setForeground(new org.eclipse.swt.graphics.Color(Display.getDefault(),
                0, 0, 255));
        desc.setText(link_text);
        //actions in click
        desc.addHyperlinkListener(new IHyperlinkListener() {
            
            @Override
            public void linkExited(HyperlinkEvent e) {
                // TODO Auto-generated method stub
                
            }
            
            @Override
            public void linkEntered(HyperlinkEvent e) {
                // TODO Auto-generated method stub
                
            }
            
            @Override
            public void linkActivated(HyperlinkEvent e) {
                PlatformUI.getWorkbench().getHelpSystem().displayHelpResource(link_href);
            }
        });
        //panel for browser engine selection
        Group container = new Group(parent, SWT.NONE);
        container.setText("Browser Engine:");
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        container.setLayout(gridLayout);
        container.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        //obtaining information about possible engine
        //and their states
        SortedMap<Integer, String> engNames = RequalityPlugin.getBrowserEngineNames();
        engState = RequalityPlugin.getBrowserEnginesState();
        int sel = getSelectedBrowserEngine();
        //current state
        Label l2 = new Label(container, SWT.NONE);
        l2.setText("Active browser engine: ");
        activeEngine = new Label(container, SWT.NONE);
        activeEngine.setText(engNames.get(RequalityPlugin.BROWSER_TYPE));
        //list of browsers
        Label l = new Label(container, SWT.NONE);
        l.setText("Browser engine: ");
        list = new Combo(container, SWT.BORDER);
        list.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        //One special element for automatic browser selection
        list.add("Automatic");
        Iterator<Integer> engIt = engState.keySet().iterator();
        int i = 1;
        //get currently selected engine
        while(engIt.hasNext()){
            int key = engIt.next();
            String adds = (engState.get(key))?" - avalible":" - not avalible";
            list.add(engNames.get(key)+adds);
            mappingIndex.put(i++, key);
            if(sel != -1 && sel == key)
                list.setText(engNames.get(key)+adds);
        }
        list.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                int index = list.getSelectionIndex();
                RequalityPreferences.this.setValid(index < 1 
                        || engState.get(mappingIndex.get(index)));
            }
        });
        if (sel == -1)
            list.setText(list.getItem(0));
        return container;
    }
    //programmatically call restart
    private void callRestart(){
        Activator.getDefault().getWorkbench().getDisplay();
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                Activator.getDefault().getWorkbench().restart();
            }
            });
    }
    //actions when new selection is performed
    @Override 
    public boolean performOk() {
        IPreferenceStore pref = Activator.getDefault().getPreferenceStore();
        int lsel = list.getSelectionIndex();
        boolean isOk = false, callrestart = false;
        if (lsel > 0){
            
            int sel = mappingIndex.get(lsel);
            if (engState.get(sel)){
                pref.setValue(BROWSER_ENGINE, sel);
                isOk = callrestart = true;
            }
        }else{
            isOk = true;
            if(lsel == 0)
            {
                pref.setValue(BROWSER_ENGINE, -1);
                callrestart = true;
            }
        }
        if(callrestart){
            if(MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Requality Preferences",
                    "You will need to restart Eclipse for the changes to take effect. Would you like to restart now?"))
                callRestart();
        }
        return isOk;
    }
    //Changing settings to default
    @Override 
    protected void performDefaults() {
        IPreferenceStore pref = Activator.getDefault().getPreferenceStore();
        pref.setValue(BROWSER_ENGINE, pref.getDefaultString(BROWSER_ENGINE));
        list.select(0);
        super.performDefaults();
    }

    public static int getSelectedBrowserEngine() {
        return (Activator.getDefault().getPreferenceStore().contains(BROWSER_ENGINE))
                ?Activator.getDefault().getPreferenceStore().getInt(BROWSER_ENGINE)
                :-1;
    }
    static{
        Activator.getDefault().getPreferenceStore().setDefault(BROWSER_ENGINE, -1);
    }
}
