/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.handlers.location.ILocationTab;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.eclipse.ui.editors.UniEditor;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class DocLocationPropertySection extends TreeNodePropertiesTab implements ILocationTab{

    private TreeViewer locations;
    private Document doc;

    public DocLocationPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));

        locations = new TreeViewer(parent, SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
        locations.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
        locations.setLabelProvider(WorkbenchLabelProvider
                .getDecoratingWorkbenchLabelProvider());
        locations.setSorter(new RequalitySorter());
        locations.setContentProvider(new ITreeContentProvider() {
            @Override
            public void inputChanged(Viewer viewer, Object oldInput,
                    Object newInput) {
            }

            @Override
            public void dispose() {
            }

            @Override
            public boolean hasChildren(Object element) {
                TreeNode node = (TreeNode) element;
                if(node instanceof Location){
                    return true;
                }
                return node.sizeChildren() > 0;
            }

            @Override
            public Object getParent(Object element) {
                return ((TreeNode) element).getParent();
            }

            @Override
            public Object[] getElements(Object inputElement) {
                return (Object[]) inputElement;
            }

            @Override
            public Object[] getChildren(Object parentElement) {
                TreeNode node = (TreeNode) parentElement;
                ArrayList<Object> result = new ArrayList<Object>();
                if (node.getType().equals(Location.TYPE_NAME)) {
                    Location l = (Location)node;
                    if(doc.equals(l.getParent()))
                        for(Requirement r : l.getRequirements())
                            result.add(r);
                }
                return result.toArray();
            }
        });
        locations.addDoubleClickListener(new IDoubleClickListener() {

            @Override
            public void doubleClick(DoubleClickEvent event) {
                IStructuredSelection selection = (IStructuredSelection) locations
                        .getSelection();
                NodeEditorInput input;
                if (selection.getFirstElement() instanceof Requirement) {
                    Requirement doc = (Requirement) selection.getFirstElement();
                    input = new NodeEditorInput(doc);
                    try {
                        IDE.openEditor(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getActivePage(), input,
                                UniEditor.ID, false);
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                    return;
                } else if (selection.getFirstElement() instanceof Location) {
                    Location loc = (Location) selection.getFirstElement();
                    input = new NodeEditorInput(loc.getParent());
                    input.setData(loc);
                } else {
                    return;
                }

                try {
                    String docId = input.getStorage().getNode()
                            .getQualifiedId();
                    IEditorReference ref = RequalityPlugin
                            .findMarkEditorForDocument(
                                    docId,
                                    TreesTracker.getResourceByNode(
                                            input.getStorage().getNode())
                                            .getProject());
                    if (ref != null && ref.getEditor(true) != null) {
                        ReqMarker ed = (ReqMarker) ref.getEditor(true);
                        ed.setTargetLocation((Location) input.getData());
                    }
                    IDE.openEditor(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getActivePage(), input,
                            ReqMarker.ID, false);
                } catch (PartInitException e) {
                    e.printStackTrace();
                } catch (CoreException e) {
                    e.printStackTrace();
                }
            }
        });
        IViewPart view = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage()
                .findView("org.eclipse.ui.views.PropertySheet");
        if (view == null)
            return;
        IWorkbenchPartSite site = view.getSite();
        MenuManager mgr = new MenuManager();
        mgr.setRemoveAllWhenShown(true);
        site.registerContextMenu(mgr, site.getSelectionProvider());
        locations.getControl().setMenu(mgr.createContextMenu(locations.getControl()));
    }
    public Document getDocument() {
        return doc;
    }

    public Location getLocation() {
        ISelection sel = locations.getSelection();
        if (!locations.getControl().isFocusControl() || doc == null ||
                !(sel instanceof IStructuredSelection)) {
            return null;
        }
        Object selo = ((IStructuredSelection) sel).getFirstElement();
        if(!(selo instanceof Location)){
            if((selo instanceof Requirement)){
                return (Location)((TreeSelection)sel).getPaths()[0].getFirstSegment();
            }else{
                return null;
            }
        }
        IStructuredSelection ssel = (IStructuredSelection) sel;
        return (Location) ssel.getFirstElement();
    }

    public void refreshLocations(){
        setTreeNode(doc);
    }

    @Override
    public void dispose() {
        super.dispose();
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        doc = toDocument(node);
        if (doc == null || locations == null
                || locations.getControl().isDisposed()) {
            return;
        }
        Set<Location> recs = new HashSet<Location>();
        for (UUID l : doc.getLocations()) {
            Location loc = doc.getLocation(l.toString());
            if (loc != null && !recs.contains(loc)) 
                recs.add(loc);
        }
        locations.setInput(recs.toArray());
        locations.expandAll();
    }

    protected Document toDocument(TreeNode node) {
        if (node instanceof Document) {
            return (Document) node;
        }

        if (node instanceof Location) {
            Location location = (Location) node;
            Document[] docs = location.getDocuments();
            if (docs.length > 0) {
                return docs[0];
            }
        }
        return null;
    }

    @Override
    public Requirement getRequirement() {
        ISelection sel = locations.getSelection();
        if (!locations.getControl().isFocusControl() || doc == null ||
                !(sel instanceof IStructuredSelection) ||
                !(((IStructuredSelection) sel).getFirstElement() instanceof Requirement)) {
            return null;
        }
        IStructuredSelection ssel = (IStructuredSelection) sel;
        return (Requirement)ssel.getFirstElement();
    }
}
