/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.transaction.ITransactionListener;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class SourcePropertySection extends AbstractPropertySection {
    private TextViewer source;
    private FontRegistry fontRegistry;

    public SourcePropertySection() {
        fontRegistry = new FontRegistry(PlatformUI.getWorkbench().getDisplay());
        fontRegistry.put("source", new FontData[] { new FontData("Monospaced",
                12, SWT.NORMAL) });
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        source = new TextViewer(parent, SWT.MULTI | SWT.READ_ONLY);
        source.getControl().setFont(fontRegistry.get("source"));
        source.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
    }
    private boolean transupdate = false;
    INodeChangeListener ncl = new INodeChangeListener() {
		@Override
		public void deleted(TreeNode source) {
		}
		
		@Override
		public void attributeChange(final TreeNode source, String what) {
			if(source.getTreeDB().getStorage() instanceof TransactionStorage){
				transupdate = true;				
			}else{
				openNode(source);
			}
		}
	};
	ITransactionListener itl = new 
			ITransactionListener() {
				
				@Override
				public void undo(TransactionStorage tStorage) {	
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void redo(TransactionStorage tStorage) {					
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void execute(TransactionStorage tStorage) {
					if(transupdate){
						openNode(target);
						transupdate = false;
					}
				}
				
				@Override
				public void _dispose() {
					// TODO Auto-generated method stub
					
				}
			};
    @Override
    public void setInput(IWorkbenchPart part, ISelection selection) {
        super.setInput(part, selection);

        TreeNode node = TreeNodePropertiesTab.toTreeNode(selection);
        if(node==null) {
            return;
        }
        openNode(node);
        
    }
    private TreeNode target;
    private void openNode(final TreeNode node) {
		SourcePropertySection.this.getPart().getSite().getShell().getDisplay().asyncExec(new Runnable() {
			@Override
			public void run() {
				openNodeSafe(node);
			}
		});
	}
    private void openNodeSafe(TreeNode node) {
    	if(target != null){
    		target.removeChangeListener(ncl);
    		target.getTreeDB().removeTransactionListener(itl);
    	}
    	if(node == null)
    		return;
    	IFile f = (IFile) TreesTracker.getResourceByNode(node);
    	if(f == null || !f.exists())
    		return;
        Writer writer = new StringWriter();
        char[] buffer = new char[1024];
        try {
            InputStream in = f.getContents();
            Reader reader = new BufferedReader(new InputStreamReader(in,
                    f.getCharset()));
            int n;
            while ((n = reader.read(buffer)) != -1) {
                writer.write(buffer, 0, n);
            }
            source.setInput(new Document(writer.toString()));
            in.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        target = node;
        target.addChangeListener(ncl);
        target.getTreeDB().addTransactionListener(itl);
	}
}
