/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.report;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.RequalityCLI;

import freemarker.template.Configuration;
import freemarker.template.DefaultObjectWrapper;

public class ReportGenerator {
    public static void clearOldImages(File f, String saveTo){
    	if(!f.exists())
			return;
		String fc = RequalityCLI.getFileContent(f);
    	RequalityCLI.deleteOldImages(fc, null, null, saveTo);
	}
    public static void generateReport(Requirement rootReq, Template templ,
            String saveTo, Map<String, Object> userData, ClassLoader baseCL)
            throws Exception {
        Configuration cfg = new Configuration();
        cfg.setObjectWrapper(new DefaultObjectWrapper());
        cfg.setDirectoryForTemplateLoading(new File(templ.getReportPath()));
        for (TemplateFile t : templ.getTemplates()) {
            File outFile = new File(saveTo + File.separator
                    + t.getOutputFileName());
            clearOldImages(outFile, saveTo+ File.separator);
            outFile.getParentFile().mkdirs();
            Writer out = new OutputStreamWriter(new FileOutputStream(outFile),
                    "UTF-8");
            try {
                Map<String, Object> data = new HashMap<String, Object>(userData);
                if (t.getDataClass().length() > 0) {
                    IReportDataSource repObject = t.instanceDataObject(baseCL);
                    data.putAll(repObject.getData());
                    data.put("report", repObject);
                } else {
                    data.put("report", new ReportTools());
                }
                data.put("reportLocation", saveTo);
                data.put("rootReq", rootReq);
                freemarker.template.Template ftl = cfg.getTemplate(
                        t.getTemplatePath(), "UTF-8");
                ftl.process(data, out);
            } catch (RuntimeException e) {
                throw e;
            } finally {
                out.close();
            }
        }
    }

    public static void generateReport(Requirement rootReq, Template templ,
            String saveTo, Map<String, Object> userData) throws Exception {
        generateReport(rootReq, templ, saveTo, userData,
                ClassLoader.getSystemClassLoader());
    }

}
